// ***************************************************************************
// *                                                                         *
// *           Aflow STEFANO CURTAROLO - Duke University 2003-2018           *
// *                                                                         *
// ***************************************************************************
// Stefano Curtarolo, 2009-2011 Duke

#ifndef _AFLOW_SPACEGROUP_CPP
#define _AFLOW_SPACEGROUP_CPP

#include "aflow.h"
using aurostd::modulus;
using aurostd::isequal;
using aurostd::isdifferent;
using aurostd::utype2string;


// ****************************************************


namespace spacegroup {
  /*
    class _spacegroup {
    public:
    // constructors/destructors
    _spacegroup(void);    // do nothing
    _spacegroup(const _spacegroup& b);    // do nothing
    ~_spacegroup();        // do nothing
    // OPERATORS                                                  // --------------------------------------
    const _spacegroup& operator=(const _spacegroup& b);             // some operators
    // CONTENT
    uint number;
    uint option;
    string stroption;
    string name;
    string sginfo;
    std::vector<_sym_op> fgroup;                                  // rotations/inversions + incell_translations operations
    std::vector<_sym_op> pgroup;                                  // rotations/inversions
   private:                                                       // ---------------------------------------
    void Free();                                                  // to free everything
    void Copy(const _spacegroup& b);                               // the flag is necessary because sometimes you need to allocate the space.
    };
  */
  std::vector<_spacegroup> vspacegroups;
}

// ****************************************************
namespace spacegroup {
  uint SpaceGroupInitialize(void) {
    if(vspacegroups.size()>=230) return vspacegroups.size(); // we have 530 entries though
    cerr << "DEBUG: spacegroup::SpaceGroupInitialize" << endl;
    string *_pstrstream;
    vector<string> tokens,database,line;
    _pstrstream=&LibrarySPACEGROUP;
    // SCAN
    database.clear();aurostd::string2tokens(*_pstrstream,database,"\n");
    // PERFORM
    for(uint isg=0;isg<database.size();isg++) {
      if(aurostd::substring2bool(database[isg],"SG#")) {
	_spacegroup spacegroup;
	uint noperations=0;
	// SG#
	aurostd::string2tokens(database[isg++],tokens," ");           // SG# tokens
	spacegroup.number=aurostd::string2utype<uint>(tokens.at(1)); // second stuff in SG# tokens
	// OPT#
	aurostd::string2tokens(database[isg++],tokens," ");           // OPT# tokens
	if(tokens.size()==1) {
	  spacegroup.stroption="";
	  spacegroup.option=0;
	} else {
	  spacegroup.stroption=tokens.at(1);
	  spacegroup.option=2; // something must find out how
	}
	// NAME#
	aurostd::string2tokens(database[isg++],tokens," ");           // OPT# tokens
	spacegroup.name=tokens.at(1);
	// SGINFO#
	aurostd::string2tokens(database[isg++],tokens," ");           // OPT# tokens
	if(tokens.size()>2) spacegroup.sginfo=tokens.at(1)+" "+tokens.at(2);
	// OPS#
	aurostd::string2tokens(database[isg++],tokens," ");           // OPT# tokens
	noperations=aurostd::string2utype<uint>(tokens.at(1)); // second stuff in OPS# tokens
	// LOAD OPERATIONS
	for(uint iops=0;iops<noperations;iops++) {
	  _sym_op symop;
	  stringstream aus;
	  symop.Uf.clear();            // 3x3  // uniques (not irreducible) operations on indices   (Uf fractional)
	  symop.flag_inversion=FALSE;  // flag if inversion
	  symop.ftau.clear();          // 3 // translation in FRACTIONAL      // FACTOR GROUP only, [0,1[
	  symop.is_fgroup=TRUE;        // bool is_fgroup
	  aus.clear();aus.str(std::string());
	  aus << database[isg++];
	  aus >> symop.Uf[1][1] >> symop.Uf[1][2] >> symop.Uf[1][3];
	  aus >> symop.Uf[2][1] >> symop.Uf[2][2] >> symop.Uf[2][3];
	  aus >> symop.Uf[3][1] >> symop.Uf[3][2] >> symop.Uf[3][3];
	  aus >> symop.ftau[1] >> symop.ftau[2] >> symop.ftau[3];
	  if(det(symop.Uf)<0) symop.flag_inversion=TRUE;
	  spacegroup.fgroup.push_back(symop);
	}
	vspacegroups.push_back(spacegroup);
      }
    }
    return vspacegroups.size();
  }
}

// ****************************************************
namespace spacegroup {
  bool SpaceGroupNumberStructure(xstructure &str) {
    SpaceGroupInitialize();
    bool sym_found;
    uint counter;
    double _eps_=0.01;
    cerr << "str.fgroup.size()=" << str.fgroup.size() << endl;
    for(uint isg=0;isg<vspacegroups.size();isg++) {
      if(vspacegroups.at(isg).fgroup.size()==str.pgroup.size()) {
	cerr << "SG=" << vspacegroups.at(isg).number << endl;
	counter=0;
	for(uint ii=0;ii<vspacegroups.at(isg).fgroup.size();ii++) {
	  sym_found=FALSE;
	  for(uint jj=0;jj<str.fgroup.size()&&!sym_found;jj++) {
	    sym_found=(identical(vspacegroups.at(isg).fgroup.at(ii).Uf,str.fgroup.at(jj).Uf,_eps_) &&
		       identical(vspacegroups.at(isg).fgroup.at(ii).ftau,str.fgroup.at(jj).ftau,_eps_));    // look in all the list of operations
	    if(sym_found) {counter++;cerr << isg << " " << ii << " " << jj << endl;}
	  }
	  //	  if(sym_found==TRUE) {counter++;cerr << "     " << counter << endl;} // nothing but discarg this
	}
	cerr << isg << " " << counter << endl;
      }
    }
    return TRUE;
  }
}

// ****************************************************
namespace spacegroup {
  bool SpaceGroupOptionRequired(uint sg) {
    if(sg==3   || sg==4   || sg==5   || sg==6   || sg==7   ||   sg==8) return TRUE;
    if(sg==9   || sg==10  || sg==11  || sg==12  || sg==13  ||  sg==14) return TRUE;
    if(sg==15  || sg==48  || sg==50  || sg==59  || sg==68  ||  sg==70) return TRUE;
    if(sg==85  || sg==86  || sg==88  || sg==125 || sg==126 || sg==129) return TRUE;
    if(sg==130 || sg==133 || sg==134 || sg==137 || sg==138 || sg==141) return TRUE;
    if(sg==142 || sg==146 || sg==148 || sg==155 || sg==160 || sg==161) return TRUE;
    if(sg==166 || sg==167 || sg==201 || sg==203 || sg==222 || sg==224) return TRUE;
    if(sg==227 || sg==228) return TRUE;
    return FALSE;
  }
}

// ****************************************************
namespace spacegroup {
  void _spacegroup::Free() {
  }
  
  // Constructors
  _spacegroup::_spacegroup(void) {
    number=0;
    option=0;
    stroption="";
    name="";
    sginfo="";
    fgroup.clear();
    pgroup.clear();
  }
  
  void _spacegroup::Copy(const _spacegroup& a) {
    // All the other stuff not set in the constructor
    number=a.number;
    option=a.option;
    stroption=a.stroption;
    name=a.name;
    sginfo=a.sginfo;
    // FGROUP ----------------------------
    fgroup.clear();
    for(uint i=0;i<a.fgroup.size();i++)
      fgroup.push_back(a.fgroup.at(i));
    // PGROUP ----------------------------
    pgroup.clear();
    for(uint i=0;i<a.pgroup.size();i++)
      pgroup.push_back(a.pgroup.at(i));
  }
  
  const _spacegroup& _spacegroup::operator=(const _spacegroup& a) {       // operator=
    if(this != &a) {
      number=a.number;
      option=a.option;
      stroption=a.stroption;
      name=a.name;
      sginfo=a.sginfo;
      // FGROUP ----------------------------
      fgroup.clear();
      for(uint i=0;i<a.fgroup.size();i++)
	fgroup.push_back(a.fgroup.at(i));
      // PGROUP ----------------------------
      pgroup.clear();
      for(uint i=0;i<a.pgroup.size();i++)
	pgroup.push_back(a.pgroup.at(i));
    }
    return *this;
  }
  
  // copy
  _spacegroup::_spacegroup(const _spacegroup& b) {
    //  Free();
    // *this=b;
    Copy(b);
  }
  
  // destructor
  _spacegroup::~_spacegroup() {
    fgroup.clear();
    pgroup.clear();
  }
} // end of namespace
  
// ****************************************************
string LibrarySPACEGROUP=" \n\
// **************************************************************** \n\
// SPACEGROUP 1  P1 \n\
SG#   1 \n\
OPT#    \n\
NAME# P1 \n\
SGINFO# 1 'P1 \n\
OPS# 1 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 2  P-1 \n\
SG#   2 \n\
OPT#    \n\
NAME# P-1 \n\
SGINFO# 2 'P-1 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 3 b P121 \n\
SG#   3 \n\
OPT#  b  \n\
NAME# P121 \n\
SGINFO# 3 'P121 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 3 c P112 \n\
SG#   3 \n\
OPT#  c  \n\
NAME# P112 \n\
SGINFO# 20003 'P112 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 3 a P211 \n\
SG#   3 \n\
OPT#  a  \n\
NAME# P211 \n\
SGINFO# 30003 'P211 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 4 b P1211 \n\
SG#   4 \n\
OPT#  b  \n\
NAME# P1211 \n\
SGINFO# 4 'P1211 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 4 c P1121 \n\
SG#   4 \n\
OPT#  c  \n\
NAME# P1121 \n\
SGINFO# 20004 'P1121 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 4 a P2111 \n\
SG#   4 \n\
OPT#  a  \n\
NAME# P2111 \n\
SGINFO# 30004 'P2111 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 5 b1 C121 \n\
SG#   5 \n\
OPT#  b1  \n\
NAME# C121 \n\
SGINFO# 5 'C121 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 5 b2 A121 \n\
SG#   5 \n\
OPT#  b2  \n\
NAME# A121 \n\
SGINFO# 12005 'A121 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 5 b3 I121 \n\
SG#   5 \n\
OPT#  b3  \n\
NAME# I121 \n\
SGINFO# 13005 'I121 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 5 c1 A112 \n\
SG#   5 \n\
OPT#  c1  \n\
NAME# A112 \n\
SGINFO# 21005 'A112 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 5 c2 B112 \n\
SG#   5 \n\
OPT#  c2  \n\
NAME# B112 \n\
SGINFO# 22005 'B112 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 5 c3 I112 \n\
SG#   5 \n\
OPT#  c3  \n\
NAME# I112 \n\
SGINFO# 23005 'I112 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 5 a1 B211 \n\
SG#   5 \n\
OPT#  a1  \n\
NAME# B211 \n\
SGINFO# 31005 'B211 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 5 a2 C211 \n\
SG#   5 \n\
OPT#  a2  \n\
NAME# C211 \n\
SGINFO# 32005 'C211 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 5 a3 I211 \n\
SG#   5 \n\
OPT#  a3  \n\
NAME# I211 \n\
SGINFO# 33005 'I211 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 6 b P1m1 \n\
SG#   6 \n\
OPT#  b  \n\
NAME# P1m1 \n\
SGINFO# 6 'P1m1 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 6 c P11m \n\
SG#   6 \n\
OPT#  c  \n\
NAME# P11m \n\
SGINFO# 20006 'P11m \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 6 a Pm11 \n\
SG#   6 \n\
OPT#  a  \n\
NAME# Pm11 \n\
SGINFO# 30006 'Pm11 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 7 b1 P1c1 \n\
SG#   7 \n\
OPT#  b1  \n\
NAME# P1c1 \n\
SGINFO# 7 'P1c1 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 7 b2 P1n1 \n\
SG#   7 \n\
OPT#  b2  \n\
NAME# P1n1 \n\
SGINFO# 12007 'P1n1 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 7 b3 P1a1 \n\
SG#   7 \n\
OPT#  b3  \n\
NAME# P1a1 \n\
SGINFO# 13007 'P1a1 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 7 c1 P11a \n\
SG#   7 \n\
OPT#  c1  \n\
NAME# P11a \n\
SGINFO# 21007 'P11a \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 7 c2 P11n \n\
SG#   7 \n\
OPT#  c2  \n\
NAME# P11n \n\
SGINFO# 22007 'P11n \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 7 c3 P11b \n\
SG#   7 \n\
OPT#  c3  \n\
NAME# P11b \n\
SGINFO# 23007 'P11b \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 7 a1 Pb11 \n\
SG#   7 \n\
OPT#  a1  \n\
NAME# Pb11 \n\
SGINFO# 31007 'Pb11 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 7 a2 Pn11 \n\
SG#   7 \n\
OPT#  a2  \n\
NAME# Pn11 \n\
SGINFO# 32007 'Pn11 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 7 a3 Pc11 \n\
SG#   7 \n\
OPT#  a3  \n\
NAME# Pc11 \n\
SGINFO# 33007 'Pc11 \n\
OPS# 2 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 b1 C1m1 \n\
SG#   8 \n\
OPT#  b1  \n\
NAME# C1m1 \n\
SGINFO# 8 'C1m1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 8 b2 A1m1 \n\
SG#   8 \n\
OPT#  b2  \n\
NAME# A1m1 \n\
SGINFO# 12008 'A1m1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 b3 I1m1 \n\
SG#   8 \n\
OPT#  b3  \n\
NAME# I1m1 \n\
SGINFO# 13008 'I1m1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 c1 A11m \n\
SG#   8 \n\
OPT#  c1  \n\
NAME# A11m \n\
SGINFO# 21008 'A11m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 c2 B11m \n\
SG#   8 \n\
OPT#  c2  \n\
NAME# B11m \n\
SGINFO# 22008 'B11m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 c3 I11m \n\
SG#   8 \n\
OPT#  c3  \n\
NAME# I11m \n\
SGINFO# 23008 'I11m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 a1 Bm11 \n\
SG#   8 \n\
OPT#  a1  \n\
NAME# Bm11 \n\
SGINFO# 31008 'Bm11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 8 a2 Cm11 \n\
SG#   8 \n\
OPT#  a2  \n\
NAME# Cm11 \n\
SGINFO# 32008 'Cm11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 8 a3 Im11 \n\
SG#   8 \n\
OPT#  a3  \n\
NAME# Im11 \n\
SGINFO# 33008 'Im11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 b1 C1c1 \n\
SG#   9 \n\
OPT#  b1  \n\
NAME# C1c1 \n\
SGINFO# 9 'C1c1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 b2 A1n1 \n\
SG#   9 \n\
OPT#  b2  \n\
NAME# A1n1 \n\
SGINFO# 12009 'A1n1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 9 b3 I1a1 \n\
SG#   9 \n\
OPT#  b3  \n\
NAME# I1a1 \n\
SGINFO# 13009 'I1a1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -b1 A1a1 \n\
SG#   9 \n\
OPT#  -b1  \n\
NAME# A1a1 \n\
SGINFO# 14009 'A1a1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -b2 C1n1 \n\
SG#   9 \n\
OPT#  -b2  \n\
NAME# C1n1 \n\
SGINFO# 15009 'C1n1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -b3 I1c1 \n\
SG#   9 \n\
OPT#  -b3  \n\
NAME# I1c1 \n\
SGINFO# 16009 'I1c1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 9 c1 A11a \n\
SG#   9 \n\
OPT#  c1  \n\
NAME# A11a \n\
SGINFO# 21009 'A11a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 c2 B11n \n\
SG#   9 \n\
OPT#  c2  \n\
NAME# B11n \n\
SGINFO# 22009 'B11n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 9 c3 I11b \n\
SG#   9 \n\
OPT#  c3  \n\
NAME# I11b \n\
SGINFO# 23009 'I11b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -c1 B11b \n\
SG#   9 \n\
OPT#  -c1  \n\
NAME# B11b \n\
SGINFO# 24009 'B11b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -c2 A11n \n\
SG#   9 \n\
OPT#  -c2  \n\
NAME# A11n \n\
SGINFO# 25009 'A11n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -c3 I11a \n\
SG#   9 \n\
OPT#  -c3  \n\
NAME# I11a \n\
SGINFO# 26009 'I11a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 a1 Bb11 \n\
SG#   9 \n\
OPT#  a1  \n\
NAME# Bb11 \n\
SGINFO# 31009 'Bb11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 a2 Cn11 \n\
SG#   9 \n\
OPT#  a2  \n\
NAME# Cn11 \n\
SGINFO# 32009 'Cn11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 a3 Ic11 \n\
SG#   9 \n\
OPT#  a3  \n\
NAME# Ic11 \n\
SGINFO# 33009 'Ic11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -a1 Cc11 \n\
SG#   9 \n\
OPT#  -a1  \n\
NAME# Cc11 \n\
SGINFO# 34009 'Cc11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -a2 Bn11 \n\
SG#   9 \n\
OPT#  -a2  \n\
NAME# Bn11 \n\
SGINFO# 35009 'Bn11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 9 -a3 Ib11 \n\
SG#   9 \n\
OPT#  -a3  \n\
NAME# Ib11 \n\
SGINFO# 36009 'Ib11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 10 b P12/m1 \n\
SG#   10 \n\
OPT#  b  \n\
NAME# P12/m1 \n\
SGINFO# 10 'P12/m1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 10 c P112/m \n\
SG#   10 \n\
OPT#  c  \n\
NAME# P112/m \n\
SGINFO# 20010 'P112/m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 10 a P2/m11 \n\
SG#   10 \n\
OPT#  a  \n\
NAME# P2/m11 \n\
SGINFO# 30010 'P2/m11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 11 b P121/m1 \n\
SG#   11 \n\
OPT#  b  \n\
NAME# P121/m1 \n\
SGINFO# 11 'P121/m1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 11 c P1121/m \n\
SG#   11 \n\
OPT#  c  \n\
NAME# P1121/m \n\
SGINFO# 20011 'P1121/m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 11 a P21/m11 \n\
SG#   11 \n\
OPT#  a  \n\
NAME# P21/m11 \n\
SGINFO# 30011 'P21/m11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 12 b1 C12/m1 \n\
SG#   12 \n\
OPT#  b1  \n\
NAME# C12/m1 \n\
SGINFO# 12 'C12/m1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 12 b2 A12/m1 \n\
SG#   12 \n\
OPT#  b2  \n\
NAME# A12/m1 \n\
SGINFO# 12012 'A12/m1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 12 b3 I12/m1 \n\
SG#   12 \n\
OPT#  b3  \n\
NAME# I12/m1 \n\
SGINFO# 13012 'I12/m1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 12 c1 A112/m \n\
SG#   12 \n\
OPT#  c1  \n\
NAME# A112/m \n\
SGINFO# 21012 'A112/m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 12 c2 B112/m \n\
SG#   12 \n\
OPT#  c2  \n\
NAME# B112/m \n\
SGINFO# 22012 'B112/m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 12 c3 I112/m \n\
SG#   12 \n\
OPT#  c3  \n\
NAME# I112/m \n\
SGINFO# 23012 'I112/m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 12 a1 B2/m11 \n\
SG#   12 \n\
OPT#  a1  \n\
NAME# B2/m11 \n\
SGINFO# 31012 'B2/m11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 12 a2 C2/m11 \n\
SG#   12 \n\
OPT#  a2  \n\
NAME# C2/m11 \n\
SGINFO# 32012 'C2/m11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 12 a3 I2/m11 \n\
SG#   12 \n\
OPT#  a3  \n\
NAME# I2/m11 \n\
SGINFO# 33012 'I2/m11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 13 b1 P12/c1 \n\
SG#   13 \n\
OPT#  b1  \n\
NAME# P12/c1 \n\
SGINFO# 13 'P12/c1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 13 b2 P12/n1 \n\
SG#   13 \n\
OPT#  b2  \n\
NAME# P12/n1 \n\
SGINFO# 12013 'P12/n1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 13 b3 P12/a1 \n\
SG#   13 \n\
OPT#  b3  \n\
NAME# P12/a1 \n\
SGINFO# 13013 'P12/a1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 13 c1 P112/a \n\
SG#   13 \n\
OPT#  c1  \n\
NAME# P112/a \n\
SGINFO# 21013 'P112/a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 13 c2 P112/n \n\
SG#   13 \n\
OPT#  c2  \n\
NAME# P112/n \n\
SGINFO# 22013 'P112/n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 13 c3 P112/b \n\
SG#   13 \n\
OPT#  c3  \n\
NAME# P112/b \n\
SGINFO# 23013 'P112/b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 13 a1 P2/b11 \n\
SG#   13 \n\
OPT#  a1  \n\
NAME# P2/b11 \n\
SGINFO# 31013 'P2/b11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 13 a2 P2/n11 \n\
SG#   13 \n\
OPT#  a2  \n\
NAME# P2/n11 \n\
SGINFO# 32013 'P2/n11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 13 a3 P2/c11 \n\
SG#   13 \n\
OPT#  a3  \n\
NAME# P2/c11 \n\
SGINFO# 33013 'P2/c11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 b1 P121/c1 \n\
SG#   14 \n\
OPT#  b1  \n\
NAME# P121/c1 \n\
SGINFO# 14 'P121/c1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 b2 P121/n1 \n\
SG#   14 \n\
OPT#  b2  \n\
NAME# P121/n1 \n\
SGINFO# 12014 'P121/n1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 b3 P121/a1 \n\
SG#   14 \n\
OPT#  b3  \n\
NAME# P121/a1 \n\
SGINFO# 13014 'P121/a1 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 14 c1 P1121/a \n\
SG#   14 \n\
OPT#  c1  \n\
NAME# P1121/a \n\
SGINFO# 21014 'P1121/a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 c2 P1121/n \n\
SG#   14 \n\
OPT#  c2  \n\
NAME# P1121/n \n\
SGINFO# 22014 'P1121/n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 c3 P1121/b \n\
SG#   14 \n\
OPT#  c3  \n\
NAME# P1121/b \n\
SGINFO# 23014 'P1121/b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 a1 P21/b11 \n\
SG#   14 \n\
OPT#  a1  \n\
NAME# P21/b11 \n\
SGINFO# 31014 'P21/b11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 14 a2 P21/n11 \n\
SG#   14 \n\
OPT#  a2  \n\
NAME# P21/n11 \n\
SGINFO# 32014 'P21/n11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 14 a3 P21/c11 \n\
SG#   14 \n\
OPT#  a3  \n\
NAME# P21/c11 \n\
SGINFO# 33014 'P21/c11 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 b1 C12/c1 \n\
SG#   15 \n\
OPT#  b1  \n\
NAME# C12/c1 \n\
SGINFO# 15 'C12/c1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 b2 A12/n1 \n\
SG#   15 \n\
OPT#  b2  \n\
NAME# A12/n1 \n\
SGINFO# 12015 'A12/n1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 15 b3 I12/a1 \n\
SG#   15 \n\
OPT#  b3  \n\
NAME# I12/a1 \n\
SGINFO# 13015 'I12/a1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -b1 A12/a1 \n\
SG#   15 \n\
OPT#  -b1  \n\
NAME# A12/a1 \n\
SGINFO# 14015 'A12/a1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -b2 C12/n1 \n\
SG#   15 \n\
OPT#  -b2  \n\
NAME# C12/n1 \n\
SGINFO# 15015 'C12/n1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -b3 I12/c1 \n\
SG#   15 \n\
OPT#  -b3  \n\
NAME# I12/c1 \n\
SGINFO# 16015 'I12/c1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 15 c1 A112/a \n\
SG#   15 \n\
OPT#  c1  \n\
NAME# A112/a \n\
SGINFO# 21015 'A112/a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 c2 B112/n \n\
SG#   15 \n\
OPT#  c2  \n\
NAME# B112/n \n\
SGINFO# 22015 'B112/n \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 15 c3 I112/b \n\
SG#   15 \n\
OPT#  c3  \n\
NAME# I112/b \n\
SGINFO# 23015 'I112/b \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -c1 B112/b \n\
SG#   15 \n\
OPT#  -c1  \n\
NAME# B112/b \n\
SGINFO# 24015 'B112/b \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -c2 A112/n \n\
SG#   15 \n\
OPT#  -c2  \n\
NAME# A112/n \n\
SGINFO# 25015 'A112/n \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -c3 I112/a \n\
SG#   15 \n\
OPT#  -c3  \n\
NAME# I112/a \n\
SGINFO# 26015 'I112/a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 a1 B2/b11 \n\
SG#   15 \n\
OPT#  a1  \n\
NAME# B2/b11 \n\
SGINFO# 31015 'B2/b11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 a2 C2/n11 \n\
SG#   15 \n\
OPT#  a2  \n\
NAME# C2/n11 \n\
SGINFO# 32015 'C2/n11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 a3 I2/c11 \n\
SG#   15 \n\
OPT#  a3  \n\
NAME# I2/c11 \n\
SGINFO# 33015 'I2/c11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -a1 C2/c11 \n\
SG#   15 \n\
OPT#  -a1  \n\
NAME# C2/c11 \n\
SGINFO# 34015 'C2/c11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -a2 B2/n11 \n\
SG#   15 \n\
OPT#  -a2  \n\
NAME# B2/n11 \n\
SGINFO# 35015 'B2/n11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 15 -a3 I2/b11 \n\
SG#   15 \n\
OPT#  -a3  \n\
NAME# I2/b11 \n\
SGINFO# 36015 'I2/b11 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 16  P222 \n\
SG#   16 \n\
OPT#    \n\
NAME# P222 \n\
SGINFO# 16 'P222 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 17  P2221 \n\
SG#   17 \n\
OPT#    \n\
NAME# P2221 \n\
SGINFO# 17 'P2221 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 17 cab P2122 \n\
SG#   17 \n\
OPT#  cab  \n\
NAME# P2122 \n\
SGINFO# 13017 'P2122 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 17 bca P2212 \n\
SG#   17 \n\
OPT#  bca  \n\
NAME# P2212 \n\
SGINFO# 15017 'P2212 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 18  P21212 \n\
SG#   18 \n\
OPT#    \n\
NAME# P21212 \n\
SGINFO# 18 'P21212 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 18 cab P22121 \n\
SG#   18 \n\
OPT#  cab  \n\
NAME# P22121 \n\
SGINFO# 13018 'P22121 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 18 bca P21221 \n\
SG#   18 \n\
OPT#  bca  \n\
NAME# P21221 \n\
SGINFO# 15018 'P21221 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 19  P212121 \n\
SG#   19 \n\
OPT#    \n\
NAME# P212121 \n\
SGINFO# 19 'P212121 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 20  C2221 \n\
SG#   20 \n\
OPT#    \n\
NAME# C2221 \n\
SGINFO# 20 'C2221 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 20 cab A2122 \n\
SG#   20 \n\
OPT#  cab  \n\
NAME# A2122 \n\
SGINFO# 13020 'A2122 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 20 bca B2212 \n\
SG#   20 \n\
OPT#  bca  \n\
NAME# B2212 \n\
SGINFO# 15020 'B2212 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 21  C222 \n\
SG#   21 \n\
OPT#    \n\
NAME# C222 \n\
SGINFO# 21 'C222 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 21 cab A222 \n\
SG#   21 \n\
OPT#  cab  \n\
NAME# A222 \n\
SGINFO# 13021 'A222 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 21 bca B222 \n\
SG#   21 \n\
OPT#  bca  \n\
NAME# B222 \n\
SGINFO# 15021 'B222 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 22  F222 \n\
SG#   22 \n\
OPT#    \n\
NAME# F222 \n\
SGINFO# 22 'F222 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 23  I222 \n\
SG#   23 \n\
OPT#    \n\
NAME# I222 \n\
SGINFO# 23 'I222 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 24  I212121 \n\
SG#   24 \n\
OPT#    \n\
NAME# I212121 \n\
SGINFO# 24 'I212121 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 25  Pmm2 \n\
SG#   25 \n\
OPT#    \n\
NAME# Pmm2 \n\
SGINFO# 25 'Pmm2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 25 cab P2mm \n\
SG#   25 \n\
OPT#  cab  \n\
NAME# P2mm \n\
SGINFO# 13025 'P2mm \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 25 bca Pm2m \n\
SG#   25 \n\
OPT#  bca  \n\
NAME# Pm2m \n\
SGINFO# 15025 'Pm2m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 26  Pmc21 \n\
SG#   26 \n\
OPT#    \n\
NAME# Pmc21 \n\
SGINFO# 26 'Pmc21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 26 ba-c Pcm21 \n\
SG#   26 \n\
OPT#  ba-c  \n\
NAME# Pcm21 \n\
SGINFO# 12026 'Pcm21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 26 cab P21ma \n\
SG#   26 \n\
OPT#  cab  \n\
NAME# P21ma \n\
SGINFO# 13026 'P21ma \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 26 -cba P21am \n\
SG#   26 \n\
OPT#  -cba  \n\
NAME# P21am \n\
SGINFO# 14026 'P21am \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 26 bca Pb21m \n\
SG#   26 \n\
OPT#  bca  \n\
NAME# Pb21m \n\
SGINFO# 15026 'Pb21m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 26 a-cb Pm21b \n\
SG#   26 \n\
OPT#  a-cb  \n\
NAME# Pm21b \n\
SGINFO# 16026 'Pm21b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 27  Pcc2 \n\
SG#   27 \n\
OPT#    \n\
NAME# Pcc2 \n\
SGINFO# 27 'Pcc2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 27 cab P2aa \n\
SG#   27 \n\
OPT#  cab  \n\
NAME# P2aa \n\
SGINFO# 13027 'P2aa \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 27 bca Pb2b \n\
SG#   27 \n\
OPT#  bca  \n\
NAME# Pb2b \n\
SGINFO# 15027 'Pb2b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 28  Pma2 \n\
SG#   28 \n\
OPT#    \n\
NAME# Pma2 \n\
SGINFO# 28 'Pma2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 28 ba-c Pbm2 \n\
SG#   28 \n\
OPT#  ba-c  \n\
NAME# Pbm2 \n\
SGINFO# 12028 'Pbm2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 28 cab P2mb \n\
SG#   28 \n\
OPT#  cab  \n\
NAME# P2mb \n\
SGINFO# 13028 'P2mb \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 28 -cba P2cm \n\
SG#   28 \n\
OPT#  -cba  \n\
NAME# P2cm \n\
SGINFO# 14028 'P2cm \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 28 bca Pc2m \n\
SG#   28 \n\
OPT#  bca  \n\
NAME# Pc2m \n\
SGINFO# 15028 'Pc2m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 28 a-cb Pm2a \n\
SG#   28 \n\
OPT#  a-cb  \n\
NAME# Pm2a \n\
SGINFO# 16028 'Pm2a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 29  Pca21 \n\
SG#   29 \n\
OPT#    \n\
NAME# Pca21 \n\
SGINFO# 29 'Pca21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 29 ba-c Pbc21 \n\
SG#   29 \n\
OPT#  ba-c  \n\
NAME# Pbc21 \n\
SGINFO# 12029 'Pbc21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 29 cab P21ab \n\
SG#   29 \n\
OPT#  cab  \n\
NAME# P21ab \n\
SGINFO# 13029 'P21ab \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 29 -cba P21ca \n\
SG#   29 \n\
OPT#  -cba  \n\
NAME# P21ca \n\
SGINFO# 14029 'P21ca \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 29 bca Pc21b \n\
SG#   29 \n\
OPT#  bca  \n\
NAME# Pc21b \n\
SGINFO# 15029 'Pc21b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 29 a-cb Pb21a \n\
SG#   29 \n\
OPT#  a-cb  \n\
NAME# Pb21a \n\
SGINFO# 16029 'Pb21a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 30  Pnc2 \n\
SG#   30 \n\
OPT#    \n\
NAME# Pnc2 \n\
SGINFO# 30 'Pnc2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 30 ba-c Pcn2 \n\
SG#   30 \n\
OPT#  ba-c  \n\
NAME# Pcn2 \n\
SGINFO# 12030 'Pcn2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 30 cab P2na \n\
SG#   30 \n\
OPT#  cab  \n\
NAME# P2na \n\
SGINFO# 13030 'P2na \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 30 -cba P2an \n\
SG#   30 \n\
OPT#  -cba  \n\
NAME# P2an \n\
SGINFO# 14030 'P2an \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 30 bca Pb2n \n\
SG#   30 \n\
OPT#  bca  \n\
NAME# Pb2n \n\
SGINFO# 15030 'Pb2n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 30 a-cb Pn2b \n\
SG#   30 \n\
OPT#  a-cb  \n\
NAME# Pn2b \n\
SGINFO# 16030 'Pn2b \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 31  Pmn21 \n\
SG#   31 \n\
OPT#    \n\
NAME# Pmn21 \n\
SGINFO# 31 'Pmn21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 31 ba-c Pnm21 \n\
SG#   31 \n\
OPT#  ba-c  \n\
NAME# Pnm21 \n\
SGINFO# 12031 'Pnm21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 31 cab P21mn \n\
SG#   31 \n\
OPT#  cab  \n\
NAME# P21mn \n\
SGINFO# 13031 'P21mn \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 31 -cba P21nm \n\
SG#   31 \n\
OPT#  -cba  \n\
NAME# P21nm \n\
SGINFO# 14031 'P21nm \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 31 bca Pn21m \n\
SG#   31 \n\
OPT#  bca  \n\
NAME# Pn21m \n\
SGINFO# 15031 'Pn21m \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 31 a-cb Pm21n \n\
SG#   31 \n\
OPT#  a-cb  \n\
NAME# Pm21n \n\
SGINFO# 16031 'Pm21n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 32  Pba2 \n\
SG#   32 \n\
OPT#    \n\
NAME# Pba2 \n\
SGINFO# 32 'Pba2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 32 cab P2cb \n\
SG#   32 \n\
OPT#  cab  \n\
NAME# P2cb \n\
SGINFO# 13032 'P2cb \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 32 bca Pc2a \n\
SG#   32 \n\
OPT#  bca  \n\
NAME# Pc2a \n\
SGINFO# 15032 'Pc2a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 33  Pna21 \n\
SG#   33 \n\
OPT#    \n\
NAME# Pna21 \n\
SGINFO# 33 'Pna21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 33 ba-c Pbn21 \n\
SG#   33 \n\
OPT#  ba-c  \n\
NAME# Pbn21 \n\
SGINFO# 12033 'Pbn21 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 33 cab P21nb \n\
SG#   33 \n\
OPT#  cab  \n\
NAME# P21nb \n\
SGINFO# 13033 'P21nb \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 33 -cba P21cn \n\
SG#   33 \n\
OPT#  -cba  \n\
NAME# P21cn \n\
SGINFO# 14033 'P21cn \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 33 bca Pc21n \n\
SG#   33 \n\
OPT#  bca  \n\
NAME# Pc21n \n\
SGINFO# 15033 'Pc21n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 33 a-cb Pn21a \n\
SG#   33 \n\
OPT#  a-cb  \n\
NAME# Pn21a \n\
SGINFO# 16033 'Pn21a \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 34  Pnn2 \n\
SG#   34 \n\
OPT#    \n\
NAME# Pnn2 \n\
SGINFO# 34 'Pnn2 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 34 cab P2nn \n\
SG#   34 \n\
OPT#  cab  \n\
NAME# P2nn \n\
SGINFO# 13034 'P2nn \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 34 bca Pn2n \n\
SG#   34 \n\
OPT#  bca  \n\
NAME# Pn2n \n\
SGINFO# 15034 'Pn2n \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 35  Cmm2 \n\
SG#   35 \n\
OPT#    \n\
NAME# Cmm2 \n\
SGINFO# 35 'Cmm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 35 cab A2mm \n\
SG#   35 \n\
OPT#  cab  \n\
NAME# A2mm \n\
SGINFO# 13035 'A2mm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 35 bca Bm2m \n\
SG#   35 \n\
OPT#  bca  \n\
NAME# Bm2m \n\
SGINFO# 15035 'Bm2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 36  Cmc21 \n\
SG#   36 \n\
OPT#    \n\
NAME# Cmc21 \n\
SGINFO# 36 'Cmc21 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 36 ba-c Ccm21 \n\
SG#   36 \n\
OPT#  ba-c  \n\
NAME# Ccm21 \n\
SGINFO# 12036 'Ccm21 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 36 cab A21ma \n\
SG#   36 \n\
OPT#  cab  \n\
NAME# A21ma \n\
SGINFO# 13036 'A21ma \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 36 -cba A21am \n\
SG#   36 \n\
OPT#  -cba  \n\
NAME# A21am \n\
SGINFO# 14036 'A21am \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 36 bca Bb21m \n\
SG#   36 \n\
OPT#  bca  \n\
NAME# Bb21m \n\
SGINFO# 15036 'Bb21m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 36 a-cb Bm21b \n\
SG#   36 \n\
OPT#  a-cb  \n\
NAME# Bm21b \n\
SGINFO# 16036 'Bm21b \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 37  Ccc2 \n\
SG#   37 \n\
OPT#    \n\
NAME# Ccc2 \n\
SGINFO# 37 'Ccc2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 37 cab A2aa \n\
SG#   37 \n\
OPT#  cab  \n\
NAME# A2aa \n\
SGINFO# 13037 'A2aa \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 37 bca Bb2b \n\
SG#   37 \n\
OPT#  bca  \n\
NAME# Bb2b \n\
SGINFO# 15037 'Bb2b \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 38  Amm2 \n\
SG#   38 \n\
OPT#    \n\
NAME# Amm2 \n\
SGINFO# 38 'Amm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 38 ba-c Bmm2 \n\
SG#   38 \n\
OPT#  ba-c  \n\
NAME# Bmm2 \n\
SGINFO# 12038 'Bmm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 38 cab B2mm \n\
SG#   38 \n\
OPT#  cab  \n\
NAME# B2mm \n\
SGINFO# 13038 'B2mm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 38 -cba C2mm \n\
SG#   38 \n\
OPT#  -cba  \n\
NAME# C2mm \n\
SGINFO# 14038 'C2mm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 38 bca Cm2m \n\
SG#   38 \n\
OPT#  bca  \n\
NAME# Cm2m \n\
SGINFO# 15038 'Cm2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 38 a-cb Am2m \n\
SG#   38 \n\
OPT#  a-cb  \n\
NAME# Am2m \n\
SGINFO# 16038 'Am2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 39  Abm2 \n\
SG#   39 \n\
OPT#    \n\
NAME# Abm2 \n\
SGINFO# 39 'Abm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 39 ba-c Bma2 \n\
SG#   39 \n\
OPT#  ba-c  \n\
NAME# Bma2 \n\
SGINFO# 12039 'Bma2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 39 cab B2cm \n\
SG#   39 \n\
OPT#  cab  \n\
NAME# B2cm \n\
SGINFO# 13039 'B2cm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 39 -cba C2mb \n\
SG#   39 \n\
OPT#  -cba  \n\
NAME# C2mb \n\
SGINFO# 14039 'C2mb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 39 bca Cm2a \n\
SG#   39 \n\
OPT#  bca  \n\
NAME# Cm2a \n\
SGINFO# 15039 'Cm2a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 39 a-cb Ac2m \n\
SG#   39 \n\
OPT#  a-cb  \n\
NAME# Ac2m \n\
SGINFO# 16039 'Ac2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 40  Ama2 \n\
SG#   40 \n\
OPT#    \n\
NAME# Ama2 \n\
SGINFO# 40 'Ama2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 40 ba-c Bbm2 \n\
SG#   40 \n\
OPT#  ba-c  \n\
NAME# Bbm2 \n\
SGINFO# 12040 'Bbm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 40 cab B2mb \n\
SG#   40 \n\
OPT#  cab  \n\
NAME# B2mb \n\
SGINFO# 13040 'B2mb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 40 -cba C2cm \n\
SG#   40 \n\
OPT#  -cba  \n\
NAME# C2cm \n\
SGINFO# 14040 'C2cm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 40 bca Cc2m \n\
SG#   40 \n\
OPT#  bca  \n\
NAME# Cc2m \n\
SGINFO# 15040 'Cc2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 40 a-cb Am2a \n\
SG#   40 \n\
OPT#  a-cb  \n\
NAME# Am2a \n\
SGINFO# 16040 'Am2a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 41  Aba2 \n\
SG#   41 \n\
OPT#    \n\
NAME# Aba2 \n\
SGINFO# 41 'Aba2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 41 ba-c Bba2 \n\
SG#   41 \n\
OPT#  ba-c  \n\
NAME# Bba2 \n\
SGINFO# 12041 'Bba2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 41 cab B2cb \n\
SG#   41 \n\
OPT#  cab  \n\
NAME# B2cb \n\
SGINFO# 13041 'B2cb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 41 -cba C2cb \n\
SG#   41 \n\
OPT#  -cba  \n\
NAME# C2cb \n\
SGINFO# 14041 'C2cb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 41 bca Cc2a \n\
SG#   41 \n\
OPT#  bca  \n\
NAME# Cc2a \n\
SGINFO# 15041 'Cc2a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 41 a-cb Ac2a \n\
SG#   41 \n\
OPT#  a-cb  \n\
NAME# Ac2a \n\
SGINFO# 16041 'Ac2a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 42  Fmm2 \n\
SG#   42 \n\
OPT#    \n\
NAME# Fmm2 \n\
SGINFO# 42 'Fmm2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 42 cab F2mm \n\
SG#   42 \n\
OPT#  cab  \n\
NAME# F2mm \n\
SGINFO# 13042 'F2mm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 42 bca Fm2m \n\
SG#   42 \n\
OPT#  bca  \n\
NAME# Fm2m \n\
SGINFO# 15042 'Fm2m \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 43  Fdd2 \n\
SG#   43 \n\
OPT#    \n\
NAME# Fdd2 \n\
SGINFO# 43 'Fdd2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.75  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 43 cab F2dd \n\
SG#   43 \n\
OPT#  cab  \n\
NAME# F2dd \n\
SGINFO# 13043 'F2dd \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.75  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 43 bca Fd2d \n\
SG#   43 \n\
OPT#  bca  \n\
NAME# Fd2d \n\
SGINFO# 15043 'Fd2d \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.75  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 44  Imm2 \n\
SG#   44 \n\
OPT#    \n\
NAME# Imm2 \n\
SGINFO# 44 'Imm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 44 cab I2mm \n\
SG#   44 \n\
OPT#  cab  \n\
NAME# I2mm \n\
SGINFO# 13044 'I2mm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 44 bca Im2m \n\
SG#   44 \n\
OPT#  bca  \n\
NAME# Im2m \n\
SGINFO# 15044 'Im2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 45  Iba2 \n\
SG#   45 \n\
OPT#    \n\
NAME# Iba2 \n\
SGINFO# 45 'Iba2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 45 cab I2cb \n\
SG#   45 \n\
OPT#  cab  \n\
NAME# I2cb \n\
SGINFO# 13045 'I2cb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 45 bca Ic2a \n\
SG#   45 \n\
OPT#  bca  \n\
NAME# Ic2a \n\
SGINFO# 15045 'Ic2a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 46  Ima2 \n\
SG#   46 \n\
OPT#    \n\
NAME# Ima2 \n\
SGINFO# 46 'Ima2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 46 ba-c Ibm2 \n\
SG#   46 \n\
OPT#  ba-c  \n\
NAME# Ibm2 \n\
SGINFO# 12046 'Ibm2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 46 cab I2mb \n\
SG#   46 \n\
OPT#  cab  \n\
NAME# I2mb \n\
SGINFO# 13046 'I2mb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 46 -cba I2cm \n\
SG#   46 \n\
OPT#  -cba  \n\
NAME# I2cm \n\
SGINFO# 14046 'I2cm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 46 bca Ic2m \n\
SG#   46 \n\
OPT#  bca  \n\
NAME# Ic2m \n\
SGINFO# 15046 'Ic2m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 46 a-cb Im2a \n\
SG#   46 \n\
OPT#  a-cb  \n\
NAME# Im2a \n\
SGINFO# 16046 'Im2a \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 47  Pmmm \n\
SG#   47 \n\
OPT#    \n\
NAME# Pmmm \n\
SGINFO# 47 'Pmmm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 48 1 Pnnn:1 \n\
SG#   48 \n\
OPT#  1  \n\
NAME# Pnnn:1 \n\
SGINFO# 48 'Pnnn:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 48 2 Pnnn:2 \n\
SG#   48 \n\
OPT#  2  \n\
NAME# Pnnn:2 \n\
SGINFO# 20048 'Pnnn:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 49  Pccm \n\
SG#   49 \n\
OPT#    \n\
NAME# Pccm \n\
SGINFO# 49 'Pccm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 49 cab Pmaa \n\
SG#   49 \n\
OPT#  cab  \n\
NAME# Pmaa \n\
SGINFO# 13049 'Pmaa \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 49 bca Pbmb \n\
SG#   49 \n\
OPT#  bca  \n\
NAME# Pbmb \n\
SGINFO# 15049 'Pbmb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 50 1 Pban:1 \n\
SG#   50 \n\
OPT#  1  \n\
NAME# Pban:1 \n\
SGINFO# 50 'Pban:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 50 2 Pban:2 \n\
SG#   50 \n\
OPT#  2  \n\
NAME# Pban:2 \n\
SGINFO# 20050 'Pban:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 50 1cab Pncb:1 \n\
SG#   50 \n\
OPT#  1cab  \n\
NAME# Pncb:1 \n\
SGINFO# 13050 'Pncb:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 50 2cab Pncb:2 \n\
SG#   50 \n\
OPT#  2cab  \n\
NAME# Pncb:2 \n\
SGINFO# 23050 'Pncb:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 50 1bca Pcna:1 \n\
SG#   50 \n\
OPT#  1bca  \n\
NAME# Pcna:1 \n\
SGINFO# 15050 'Pcna:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 50 2bca Pcna:2 \n\
SG#   50 \n\
OPT#  2bca  \n\
NAME# Pcna:2 \n\
SGINFO# 25050 'Pcna:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 51  Pmma \n\
SG#   51 \n\
OPT#    \n\
NAME# Pmma \n\
SGINFO# 51 'Pmma \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 51 ba-c Pmmb \n\
SG#   51 \n\
OPT#  ba-c  \n\
NAME# Pmmb \n\
SGINFO# 12051 'Pmmb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 51 cab Pbmm \n\
SG#   51 \n\
OPT#  cab  \n\
NAME# Pbmm \n\
SGINFO# 13051 'Pbmm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 51 -cba Pcmm \n\
SG#   51 \n\
OPT#  -cba  \n\
NAME# Pcmm \n\
SGINFO# 14051 'Pcmm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 51 bca Pmcm \n\
SG#   51 \n\
OPT#  bca  \n\
NAME# Pmcm \n\
SGINFO# 15051 'Pmcm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 51 a-cb Pmam \n\
SG#   51 \n\
OPT#  a-cb  \n\
NAME# Pmam \n\
SGINFO# 16051 'Pmam \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 52  Pnna \n\
SG#   52 \n\
OPT#    \n\
NAME# Pnna \n\
SGINFO# 52 'Pnna \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 52 ba-c Pnnb \n\
SG#   52 \n\
OPT#  ba-c  \n\
NAME# Pnnb \n\
SGINFO# 12052 'Pnnb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 52 cab Pbnn \n\
SG#   52 \n\
OPT#  cab  \n\
NAME# Pbnn \n\
SGINFO# 13052 'Pbnn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 52 -cba Pcnn \n\
SG#   52 \n\
OPT#  -cba  \n\
NAME# Pcnn \n\
SGINFO# 14052 'Pcnn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 52 bca Pncn \n\
SG#   52 \n\
OPT#  bca  \n\
NAME# Pncn \n\
SGINFO# 15052 'Pncn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 52 a-cb Pnan \n\
SG#   52 \n\
OPT#  a-cb  \n\
NAME# Pnan \n\
SGINFO# 16052 'Pnan \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 53  Pmna \n\
SG#   53 \n\
OPT#    \n\
NAME# Pmna \n\
SGINFO# 53 'Pmna \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 53 ba-c Pnmb \n\
SG#   53 \n\
OPT#  ba-c  \n\
NAME# Pnmb \n\
SGINFO# 12053 'Pnmb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 53 cab Pbmn \n\
SG#   53 \n\
OPT#  cab  \n\
NAME# Pbmn \n\
SGINFO# 13053 'Pbmn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 53 -cba Pcnm \n\
SG#   53 \n\
OPT#  -cba  \n\
NAME# Pcnm \n\
SGINFO# 14053 'Pcnm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 53 bca Pncm \n\
SG#   53 \n\
OPT#  bca  \n\
NAME# Pncm \n\
SGINFO# 15053 'Pncm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 53 a-cb Pman \n\
SG#   53 \n\
OPT#  a-cb  \n\
NAME# Pman \n\
SGINFO# 16053 'Pman \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 54  Pcca \n\
SG#   54 \n\
OPT#    \n\
NAME# Pcca \n\
SGINFO# 54 'Pcca \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 54 ba-c Pccb \n\
SG#   54 \n\
OPT#  ba-c  \n\
NAME# Pccb \n\
SGINFO# 12054 'Pccb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 54 cab Pbaa \n\
SG#   54 \n\
OPT#  cab  \n\
NAME# Pbaa \n\
SGINFO# 13054 'Pbaa \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 54 -cba Pcaa \n\
SG#   54 \n\
OPT#  -cba  \n\
NAME# Pcaa \n\
SGINFO# 14054 'Pcaa \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 54 bca Pbcb \n\
SG#   54 \n\
OPT#  bca  \n\
NAME# Pbcb \n\
SGINFO# 15054 'Pbcb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 54 a-cb Pbab \n\
SG#   54 \n\
OPT#  a-cb  \n\
NAME# Pbab \n\
SGINFO# 16054 'Pbab \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 55  Pbam \n\
SG#   55 \n\
OPT#    \n\
NAME# Pbam \n\
SGINFO# 55 'Pbam \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 55 cab Pmcb \n\
SG#   55 \n\
OPT#  cab  \n\
NAME# Pmcb \n\
SGINFO# 13055 'Pmcb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 55 bca Pcma \n\
SG#   55 \n\
OPT#  bca  \n\
NAME# Pcma \n\
SGINFO# 15055 'Pcma \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 56  Pccn \n\
SG#   56 \n\
OPT#    \n\
NAME# Pccn \n\
SGINFO# 56 'Pccn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 56 cab Pnaa \n\
SG#   56 \n\
OPT#  cab  \n\
NAME# Pnaa \n\
SGINFO# 13056 'Pnaa \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 56 bca Pbnb \n\
SG#   56 \n\
OPT#  bca  \n\
NAME# Pbnb \n\
SGINFO# 15056 'Pbnb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 57  Pbcm \n\
SG#   57 \n\
OPT#    \n\
NAME# Pbcm \n\
SGINFO# 57 'Pbcm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 57 ba-c Pcam \n\
SG#   57 \n\
OPT#  ba-c  \n\
NAME# Pcam \n\
SGINFO# 12057 'Pcam \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 57 cab Pmca \n\
SG#   57 \n\
OPT#  cab  \n\
NAME# Pmca \n\
SGINFO# 13057 'Pmca \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 57 -cba Pmab \n\
SG#   57 \n\
OPT#  -cba  \n\
NAME# Pmab \n\
SGINFO# 14057 'Pmab \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 57 bca Pbma \n\
SG#   57 \n\
OPT#  bca  \n\
NAME# Pbma \n\
SGINFO# 15057 'Pbma \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 57 a-cb Pcmb \n\
SG#   57 \n\
OPT#  a-cb  \n\
NAME# Pcmb \n\
SGINFO# 16057 'Pcmb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 58  Pnnm \n\
SG#   58 \n\
OPT#    \n\
NAME# Pnnm \n\
SGINFO# 58 'Pnnm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 58 cab Pmnn \n\
SG#   58 \n\
OPT#  cab  \n\
NAME# Pmnn \n\
SGINFO# 13058 'Pmnn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 58 bca Pnmn \n\
SG#   58 \n\
OPT#  bca  \n\
NAME# Pnmn \n\
SGINFO# 15058 'Pnmn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 59 1 Pmmn:1 \n\
SG#   59 \n\
OPT#  1  \n\
NAME# Pmmn:1 \n\
SGINFO# 59 'Pmmn:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 59 2 Pmmn:2 \n\
SG#   59 \n\
OPT#  2  \n\
NAME# Pmmn:2 \n\
SGINFO# 20059 'Pmmn:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 59 1cab Pnmm:1 \n\
SG#   59 \n\
OPT#  1cab  \n\
NAME# Pnmm:1 \n\
SGINFO# 13059 'Pnmm:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 59 2cab Pnmm:2 \n\
SG#   59 \n\
OPT#  2cab  \n\
NAME# Pnmm:2 \n\
SGINFO# 23059 'Pnmm:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 59 1bca Pmnm:1 \n\
SG#   59 \n\
OPT#  1bca  \n\
NAME# Pmnm:1 \n\
SGINFO# 15059 'Pmnm:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 59 2bca Pmnm:2 \n\
SG#   59 \n\
OPT#  2bca  \n\
NAME# Pmnm:2 \n\
SGINFO# 25059 'Pmnm:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 60  Pbcn \n\
SG#   60 \n\
OPT#    \n\
NAME# Pbcn \n\
SGINFO# 60 'Pbcn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 60 ba-c Pcan \n\
SG#   60 \n\
OPT#  ba-c  \n\
NAME# Pcan \n\
SGINFO# 12060 'Pcan \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 60 cab Pnca \n\
SG#   60 \n\
OPT#  cab  \n\
NAME# Pnca \n\
SGINFO# 13060 'Pnca \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 60 -cba Pnab \n\
SG#   60 \n\
OPT#  -cba  \n\
NAME# Pnab \n\
SGINFO# 14060 'Pnab \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 60 bca Pbna \n\
SG#   60 \n\
OPT#  bca  \n\
NAME# Pbna \n\
SGINFO# 15060 'Pbna \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 60 a-cb Pcnb \n\
SG#   60 \n\
OPT#  a-cb  \n\
NAME# Pcnb \n\
SGINFO# 16060 'Pcnb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 61  Pbca \n\
SG#   61 \n\
OPT#    \n\
NAME# Pbca \n\
SGINFO# 61 'Pbca \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 61 ba-c Pcab \n\
SG#   61 \n\
OPT#  ba-c  \n\
NAME# Pcab \n\
SGINFO# 12061 'Pcab \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 62  Pnma \n\
SG#   62 \n\
OPT#    \n\
NAME# Pnma \n\
SGINFO# 62 'Pnma \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 62 ba-c Pmnb \n\
SG#   62 \n\
OPT#  ba-c  \n\
NAME# Pmnb \n\
SGINFO# 12062 'Pmnb \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 62 cab Pbnm \n\
SG#   62 \n\
OPT#  cab  \n\
NAME# Pbnm \n\
SGINFO# 13062 'Pbnm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 62 -cba Pcmn \n\
SG#   62 \n\
OPT#  -cba  \n\
NAME# Pcmn \n\
SGINFO# 14062 'Pcmn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 62 bca Pmcn \n\
SG#   62 \n\
OPT#  bca  \n\
NAME# Pmcn \n\
SGINFO# 15062 'Pmcn \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 62 a-cb Pnam \n\
SG#   62 \n\
OPT#  a-cb  \n\
NAME# Pnam \n\
SGINFO# 16062 'Pnam \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 63  Cmcm \n\
SG#   63 \n\
OPT#    \n\
NAME# Cmcm \n\
SGINFO# 63 'Cmcm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 63 ba-c Ccmm \n\
SG#   63 \n\
OPT#  ba-c  \n\
NAME# Ccmm \n\
SGINFO# 12063 'Ccmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 63 cab Amma \n\
SG#   63 \n\
OPT#  cab  \n\
NAME# Amma \n\
SGINFO# 13063 'Amma \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 63 -cba Amam \n\
SG#   63 \n\
OPT#  -cba  \n\
NAME# Amam \n\
SGINFO# 14063 'Amam \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 63 bca Bbmm \n\
SG#   63 \n\
OPT#  bca  \n\
NAME# Bbmm \n\
SGINFO# 15063 'Bbmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 63 a-cb Bmmb \n\
SG#   63 \n\
OPT#  a-cb  \n\
NAME# Bmmb \n\
SGINFO# 16063 'Bmmb \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 64  Cmca \n\
SG#   64 \n\
OPT#    \n\
NAME# Cmca \n\
SGINFO# 64 'Cmca \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 64 ba-c Ccmb \n\
SG#   64 \n\
OPT#  ba-c  \n\
NAME# Ccmb \n\
SGINFO# 12064 'Ccmb \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 64 cab Abma \n\
SG#   64 \n\
OPT#  cab  \n\
NAME# Abma \n\
SGINFO# 13064 'Abma \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 64 -cba Acam \n\
SG#   64 \n\
OPT#  -cba  \n\
NAME# Acam \n\
SGINFO# 14064 'Acam \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 64 bca Bbcm \n\
SG#   64 \n\
OPT#  bca  \n\
NAME# Bbcm \n\
SGINFO# 15064 'Bbcm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 64 a-cb Bmab \n\
SG#   64 \n\
OPT#  a-cb  \n\
NAME# Bmab \n\
SGINFO# 16064 'Bmab \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 65  Cmmm \n\
SG#   65 \n\
OPT#    \n\
NAME# Cmmm \n\
SGINFO# 65 'Cmmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 65 cab Ammm \n\
SG#   65 \n\
OPT#  cab  \n\
NAME# Ammm \n\
SGINFO# 13065 'Ammm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 65 bca Bmmm \n\
SG#   65 \n\
OPT#  bca  \n\
NAME# Bmmm \n\
SGINFO# 15065 'Bmmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 66  Cccm \n\
SG#   66 \n\
OPT#    \n\
NAME# Cccm \n\
SGINFO# 66 'Cccm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 66 cab Amaa \n\
SG#   66 \n\
OPT#  cab  \n\
NAME# Amaa \n\
SGINFO# 13066 'Amaa \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 66 bca Bbmb \n\
SG#   66 \n\
OPT#  bca  \n\
NAME# Bbmb \n\
SGINFO# 15066 'Bbmb \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 67  Cmma \n\
SG#   67 \n\
OPT#    \n\
NAME# Cmma \n\
SGINFO# 67 'Cmma \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 67 ba-c Cmmb \n\
SG#   67 \n\
OPT#  ba-c  \n\
NAME# Cmmb \n\
SGINFO# 12067 'Cmmb \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 67 cab Abmm \n\
SG#   67 \n\
OPT#  cab  \n\
NAME# Abmm \n\
SGINFO# 13067 'Abmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 67 -cba Acmm \n\
SG#   67 \n\
OPT#  -cba  \n\
NAME# Acmm \n\
SGINFO# 14067 'Acmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 67 bca Bmcm \n\
SG#   67 \n\
OPT#  bca  \n\
NAME# Bmcm \n\
SGINFO# 15067 'Bmcm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 67 a-cb Bmam \n\
SG#   67 \n\
OPT#  a-cb  \n\
NAME# Bmam \n\
SGINFO# 16067 'Bmam \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 1 Ccca:1 \n\
SG#   68 \n\
OPT#  1  \n\
NAME# Ccca:1 \n\
SGINFO# 68 'Ccca:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 68 2 Ccca:2 \n\
SG#   68 \n\
OPT#  2  \n\
NAME# Ccca:2 \n\
SGINFO# 20068 'Ccca:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 68 1ba-c Cccb:1 \n\
SG#   68 \n\
OPT#  1ba-c  \n\
NAME# Cccb:1 \n\
SGINFO# 12068 'Cccb:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 68 2ba-c Cccb:2 \n\
SG#   68 \n\
OPT#  2ba-c  \n\
NAME# Cccb:2 \n\
SGINFO# 22068 'Cccb:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 68 1cab Abaa:1 \n\
SG#   68 \n\
OPT#  1cab  \n\
NAME# Abaa:1 \n\
SGINFO# 13068 'Abaa:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 2cab Abaa:2 \n\
SG#   68 \n\
OPT#  2cab  \n\
NAME# Abaa:2 \n\
SGINFO# 23068 'Abaa:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 1-cba Acaa:1 \n\
SG#   68 \n\
OPT#  1-cba  \n\
NAME# Acaa:1 \n\
SGINFO# 14068 'Acaa:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 2-cba Acaa:2 \n\
SG#   68 \n\
OPT#  2-cba  \n\
NAME# Acaa:2 \n\
SGINFO# 24068 'Acaa:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 68 1bca Bbcb:1 \n\
SG#   68 \n\
OPT#  1bca  \n\
NAME# Bbcb:1 \n\
SGINFO# 15068 'Bbcb:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 2bca Bbcb:2 \n\
SG#   68 \n\
OPT#  2bca  \n\
NAME# Bbcb:2 \n\
SGINFO# 25068 'Bbcb:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 1a-cb Bbab:1 \n\
SG#   68 \n\
OPT#  1a-cb  \n\
NAME# Bbab:1 \n\
SGINFO# 16068 'Bbab:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 68 2a-cb Bbab:2 \n\
SG#   68 \n\
OPT#  2a-cb  \n\
NAME# Bbab:2 \n\
SGINFO# 26068 'Bbab:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 69  Fmmm \n\
SG#   69 \n\
OPT#    \n\
NAME# Fmmm \n\
SGINFO# 69 'Fmmm \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 70 1 Fddd:1 \n\
SG#   70 \n\
OPT#  1  \n\
NAME# Fddd:1 \n\
SGINFO# 70 'Fddd:1 \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.75  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 70 2 Fddd:2 \n\
SG#   70 \n\
OPT#  2  \n\
NAME# Fddd:2 \n\
SGINFO# 20070 'Fddd:2 \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.25  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.00  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.00  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.75  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.50  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.25  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.00  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.75  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.50  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 71  Immm \n\
SG#   71 \n\
OPT#    \n\
NAME# Immm \n\
SGINFO# 71 'Immm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 72  Ibam \n\
SG#   72 \n\
OPT#    \n\
NAME# Ibam \n\
SGINFO# 72 'Ibam \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 72 cab Imcb \n\
SG#   72 \n\
OPT#  cab  \n\
NAME# Imcb \n\
SGINFO# 13072 'Imcb \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 72 bca Icma \n\
SG#   72 \n\
OPT#  bca  \n\
NAME# Icma \n\
SGINFO# 15072 'Icma \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 73  Ibca \n\
SG#   73 \n\
OPT#    \n\
NAME# Ibca \n\
SGINFO# 73 'Ibca \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 73 ba-c Icab \n\
SG#   73 \n\
OPT#  ba-c  \n\
NAME# Icab \n\
SGINFO# 12073 'Icab \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 74  Imma \n\
SG#   74 \n\
OPT#    \n\
NAME# Imma \n\
SGINFO# 74 'Imma \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 74 ba-c Immb \n\
SG#   74 \n\
OPT#  ba-c  \n\
NAME# Immb \n\
SGINFO# 12074 'Immb \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 74 cab Ibmm \n\
SG#   74 \n\
OPT#  cab  \n\
NAME# Ibmm \n\
SGINFO# 13074 'Ibmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 74 -cba Icmm \n\
SG#   74 \n\
OPT#  -cba  \n\
NAME# Icmm \n\
SGINFO# 14074 'Icmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 74 bca Imcm \n\
SG#   74 \n\
OPT#  bca  \n\
NAME# Imcm \n\
SGINFO# 15074 'Imcm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 74 a-cb Imam \n\
SG#   74 \n\
OPT#  a-cb  \n\
NAME# Imam \n\
SGINFO# 16074 'Imam \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 75  P4 \n\
SG#   75 \n\
OPT#    \n\
NAME# P4 \n\
SGINFO# 75 'P4 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 76  P41 \n\
SG#   76 \n\
OPT#    \n\
NAME# P41 \n\
SGINFO# 76 'P41 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 77  P42 \n\
SG#   77 \n\
OPT#    \n\
NAME# P42 \n\
SGINFO# 77 'P42 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 78  P43 \n\
SG#   78 \n\
OPT#    \n\
NAME# P43 \n\
SGINFO# 78 'P43 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 79  I4 \n\
SG#   79 \n\
OPT#    \n\
NAME# I4 \n\
SGINFO# 79 'I4 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 80  I41 \n\
SG#   80 \n\
OPT#    \n\
NAME# I41 \n\
SGINFO# 80 'I41 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 81  P-4 \n\
SG#   81 \n\
OPT#    \n\
NAME# P-4 \n\
SGINFO# 81 'P-4 \n\
OPS# 4 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 82  I-4 \n\
SG#   82 \n\
OPT#    \n\
NAME# I-4 \n\
SGINFO# 82 'I-4 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 83  P4/m \n\
SG#   83 \n\
OPT#    \n\
NAME# P4/m \n\
SGINFO# 83 'P4/m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 84  P42/m \n\
SG#   84 \n\
OPT#    \n\
NAME# P42/m \n\
SGINFO# 84 'P42/m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 85 1 P4/n:1 \n\
SG#   85 \n\
OPT#  1  \n\
NAME# P4/n:1 \n\
SGINFO# 85 'P4/n:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 85 2 P4/n:2 \n\
SG#   85 \n\
OPT#  2  \n\
NAME# P4/n:2 \n\
SGINFO# 20085 'P4/n:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 86 1 P42/n:1 \n\
SG#   86 \n\
OPT#  1  \n\
NAME# P42/n:1 \n\
SGINFO# 86 'P42/n:1 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 86 2 P42/n:2 \n\
SG#   86 \n\
OPT#  2  \n\
NAME# P42/n:2 \n\
SGINFO# 20086 'P42/n:2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 87  I4/m \n\
SG#   87 \n\
OPT#    \n\
NAME# I4/m \n\
SGINFO# 87 'I4/m \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 88 1 I41/a:1 \n\
SG#   88 \n\
OPT#  1  \n\
NAME# I41/a:1 \n\
SGINFO# 88 'I41/a:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 88 2 I41/a:2 \n\
SG#   88 \n\
OPT#  2  \n\
NAME# I41/a:2 \n\
SGINFO# 20088 'I41/a:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.75  0.25  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.25  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 89  P422 \n\
SG#   89 \n\
OPT#    \n\
NAME# P422 \n\
SGINFO# 89 'P422 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 90  P4212 \n\
SG#   90 \n\
OPT#    \n\
NAME# P4212 \n\
SGINFO# 90 'P4212 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 91  P4122 \n\
SG#   91 \n\
OPT#    \n\
NAME# P4122 \n\
SGINFO# 91 'P4122 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 92  P41212 \n\
SG#   92 \n\
OPT#    \n\
NAME# P41212 \n\
SGINFO# 92 'P41212 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 93  P4222 \n\
SG#   93 \n\
OPT#    \n\
NAME# P4222 \n\
SGINFO# 93 'P4222 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 94  P42212 \n\
SG#   94 \n\
OPT#    \n\
NAME# P42212 \n\
SGINFO# 94 'P42212 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 95  P4322 \n\
SG#   95 \n\
OPT#    \n\
NAME# P4322 \n\
SGINFO# 95 'P4322 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 96  P43212 \n\
SG#   96 \n\
OPT#    \n\
NAME# P43212 \n\
SGINFO# 96 'P43212 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 97  I422 \n\
SG#   97 \n\
OPT#    \n\
NAME# I422 \n\
SGINFO# 97 'I422 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 98  I4122 \n\
SG#   98 \n\
OPT#    \n\
NAME# I4122 \n\
SGINFO# 98 'I4122 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 99  P4mm \n\
SG#   99 \n\
OPT#    \n\
NAME# P4mm \n\
SGINFO# 99 'P4mm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 100  P4bm \n\
SG#   100 \n\
OPT#    \n\
NAME# P4bm \n\
SGINFO# 100 'P4bm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 101  P42cm \n\
SG#   101 \n\
OPT#    \n\
NAME# P42cm \n\
SGINFO# 101 'P42cm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 102  P42nm \n\
SG#   102 \n\
OPT#    \n\
NAME# P42nm \n\
SGINFO# 102 'P42nm \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 103  P4cc \n\
SG#   103 \n\
OPT#    \n\
NAME# P4cc \n\
SGINFO# 103 'P4cc \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 104  P4nc \n\
SG#   104 \n\
OPT#    \n\
NAME# P4nc \n\
SGINFO# 104 'P4nc \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 105  P42mc \n\
SG#   105 \n\
OPT#    \n\
NAME# P42mc \n\
SGINFO# 105 'P42mc \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 106  P42bc \n\
SG#   106 \n\
OPT#    \n\
NAME# P42bc \n\
SGINFO# 106 'P42bc \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 107  I4mm \n\
SG#   107 \n\
OPT#    \n\
NAME# I4mm \n\
SGINFO# 107 'I4mm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 108  I4cm \n\
SG#   108 \n\
OPT#    \n\
NAME# I4cm \n\
SGINFO# 108 'I4cm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 109  I41md \n\
SG#   109 \n\
OPT#    \n\
NAME# I41md \n\
SGINFO# 109 'I41md \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 110  I41cd \n\
SG#   110 \n\
OPT#    \n\
NAME# I41cd \n\
SGINFO# 110 'I41cd \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 111  P-42m \n\
SG#   111 \n\
OPT#    \n\
NAME# P-42m \n\
SGINFO# 111 'P-42m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 112  P-42c \n\
SG#   112 \n\
OPT#    \n\
NAME# P-42c \n\
SGINFO# 112 'P-42c \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 113  P-421m \n\
SG#   113 \n\
OPT#    \n\
NAME# P-421m \n\
SGINFO# 113 'P-421m \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 114  P-421c \n\
SG#   114 \n\
OPT#    \n\
NAME# P-421c \n\
SGINFO# 114 'P-421c \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 115  P-4m2 \n\
SG#   115 \n\
OPT#    \n\
NAME# P-4m2 \n\
SGINFO# 115 'P-4m2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 116  P-4c2 \n\
SG#   116 \n\
OPT#    \n\
NAME# P-4c2 \n\
SGINFO# 116 'P-4c2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 117  P-4b2 \n\
SG#   117 \n\
OPT#    \n\
NAME# P-4b2 \n\
SGINFO# 117 'P-4b2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 118  P-4n2 \n\
SG#   118 \n\
OPT#    \n\
NAME# P-4n2 \n\
SGINFO# 118 'P-4n2 \n\
OPS# 8 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 119  I-4m2 \n\
SG#   119 \n\
OPT#    \n\
NAME# I-4m2 \n\
SGINFO# 119 'I-4m2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 120  I-4c2 \n\
SG#   120 \n\
OPT#    \n\
NAME# I-4c2 \n\
SGINFO# 120 'I-4c2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 121  I-42m \n\
SG#   121 \n\
OPT#    \n\
NAME# I-42m \n\
SGINFO# 121 'I-42m \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 122  I-42d \n\
SG#   122 \n\
OPT#    \n\
NAME# I-42d \n\
SGINFO# 122 'I-42d \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 123  P4/mmm \n\
SG#   123 \n\
OPT#    \n\
NAME# P4/mmm \n\
SGINFO# 123 'P4/mmm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 124  P4/mcc \n\
SG#   124 \n\
OPT#    \n\
NAME# P4/mcc \n\
SGINFO# 124 'P4/mcc \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 125 1 P4/nbm:1 \n\
SG#   125 \n\
OPT#  1  \n\
NAME# P4/nbm:1 \n\
SGINFO# 125 'P4/nbm:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 125 2 P4/nbm:2 \n\
SG#   125 \n\
OPT#  2  \n\
NAME# P4/nbm:2 \n\
SGINFO# 20125 'P4/nbm:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 126 1 P4/nnc:1 \n\
SG#   126 \n\
OPT#  1  \n\
NAME# P4/nnc:1 \n\
SGINFO# 126 'P4/nnc:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 126 2 P4/nnc:2 \n\
SG#   126 \n\
OPT#  2  \n\
NAME# P4/nnc:2 \n\
SGINFO# 20126 'P4/nnc:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 127  P4/mbm \n\
SG#   127 \n\
OPT#    \n\
NAME# P4/mbm \n\
SGINFO# 127 'P4/mbm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 128  P4/mnc \n\
SG#   128 \n\
OPT#    \n\
NAME# P4/mnc \n\
SGINFO# 128 'P4/mnc \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 129 1 P4/nmm:1 \n\
SG#   129 \n\
OPT#  1  \n\
NAME# P4/nmm:1 \n\
SGINFO# 129 'P4/nmm:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 129 2 P4/nmm:2 \n\
SG#   129 \n\
OPT#  2  \n\
NAME# P4/nmm:2 \n\
SGINFO# 20129 'P4/nmm:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 130 1 P4/ncc:1 \n\
SG#   130 \n\
OPT#  1  \n\
NAME# P4/ncc:1 \n\
SGINFO# 130 'P4/ncc:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 130 2 P4/ncc:2 \n\
SG#   130 \n\
OPT#  2  \n\
NAME# P4/ncc:2 \n\
SGINFO# 20130 'P4/ncc:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 131  P42/mmc \n\
SG#   131 \n\
OPT#    \n\
NAME# P42/mmc \n\
SGINFO# 131 'P42/mmc \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 132  P42/mcm \n\
SG#   132 \n\
OPT#    \n\
NAME# P42/mcm \n\
SGINFO# 132 'P42/mcm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 133 1 P42/nbc:1 \n\
SG#   133 \n\
OPT#  1  \n\
NAME# P42/nbc:1 \n\
SGINFO# 133 'P42/nbc:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 133 2 P42/nbc:2 \n\
SG#   133 \n\
OPT#  2  \n\
NAME# P42/nbc:2 \n\
SGINFO# 20133 'P42/nbc:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 134 1 P42/nnm:1 \n\
SG#   134 \n\
OPT#  1  \n\
NAME# P42/nnm:1 \n\
SGINFO# 134 'P42/nnm:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 134 2 P42/nnm:2 \n\
SG#   134 \n\
OPT#  2  \n\
NAME# P42/nnm:2 \n\
SGINFO# 20134 'P42/nnm:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 135  P42/mbc \n\
SG#   135 \n\
OPT#    \n\
NAME# P42/mbc \n\
SGINFO# 135 'P42/mbc \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 136  P42/mnm \n\
SG#   136 \n\
OPT#    \n\
NAME# P42/mnm \n\
SGINFO# 136 'P42/mnm \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 137 1 P42/nmc:1 \n\
SG#   137 \n\
OPT#  1  \n\
NAME# P42/nmc:1 \n\
SGINFO# 137 'P42/nmc:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 137 2 P42/nmc:2 \n\
SG#   137 \n\
OPT#  2  \n\
NAME# P42/nmc:2 \n\
SGINFO# 20137 'P42/nmc:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 138 1 P42/ncm:1 \n\
SG#   138 \n\
OPT#  1  \n\
NAME# P42/ncm:1 \n\
SGINFO# 138 'P42/ncm:1 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 138 2 P42/ncm:2 \n\
SG#   138 \n\
OPT#  2  \n\
NAME# P42/ncm:2 \n\
SGINFO# 20138 'P42/ncm:2 \n\
OPS# 16 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 139  I4/mmm \n\
SG#   139 \n\
OPT#    \n\
NAME# I4/mmm \n\
SGINFO# 139 'I4/mmm \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 140  I4/mcm \n\
SG#   140 \n\
OPT#    \n\
NAME# I4/mcm \n\
SGINFO# 140 'I4/mcm \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 141 1 I41/amd:1 \n\
SG#   141 \n\
OPT#  1  \n\
NAME# I41/amd:1 \n\
SGINFO# 141 'I41/amd:1 \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 141 2 I41/amd:2 \n\
SG#   141 \n\
OPT#  2  \n\
NAME# I41/amd:2 \n\
SGINFO# 20141 'I41/amd:2 \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.25  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.25  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.75  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.25  0.25  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 142 1 I41/acd:1 \n\
SG#   142 \n\
OPT#  1  \n\
NAME# I41/acd:1 \n\
SGINFO# 142 'I41/acd:1 \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 142 2 I41/acd:2 \n\
SG#   142 \n\
OPT#  2  \n\
NAME# I41/acd:2 \n\
SGINFO# 20142 'I41/acd:2 \n\
OPS# 32 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.75  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.75  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.25  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.25  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.25  0.25  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 143  P3 \n\
SG#   143 \n\
OPT#    \n\
NAME# P3 \n\
SGINFO# 143 'P3 \n\
OPS# 3 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 144  P31 \n\
SG#   144 \n\
OPT#    \n\
NAME# P31 \n\
SGINFO# 144 'P31 \n\
OPS# 3 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 145  P32 \n\
SG#   145 \n\
OPT#    \n\
NAME# P32 \n\
SGINFO# 145 'P32 \n\
OPS# 3 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
// **************************************************************** \n\
// SPACEGROUP 146 H R3:H \n\
SG#   146 \n\
OPT#  H  \n\
NAME# R3:H \n\
SGINFO# 146 'R3:H \n\
OPS# 9 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 146 R R3:R \n\
SG#   146 \n\
OPT#  R  \n\
NAME# R3:R \n\
SGINFO# 20146 'R3:R \n\
OPS# 3 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 147  P-3 \n\
SG#   147 \n\
OPT#    \n\
NAME# P-3 \n\
SGINFO# 147 'P-3 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 148 H R-3:H \n\
SG#   148 \n\
OPT#  H  \n\
NAME# R-3:H \n\
SGINFO# 148 'R-3:H \n\
OPS# 18 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  0  0  0 -1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  0  1  0 -1  1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  1 -1  0  1  0  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  0  0  0 -1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  0  1  0 -1  1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  1 -1  0  1  0  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 148 R R-3:R \n\
SG#   148 \n\
OPT#  R  \n\
NAME# R-3:R \n\
SGINFO# 20148 'R-3:R \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 149  P312 \n\
SG#   149 \n\
OPT#    \n\
NAME# P312 \n\
SGINFO# 149 'P312 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 150  P321 \n\
SG#   150 \n\
OPT#    \n\
NAME# P321 \n\
SGINFO# 150 'P321 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 151  P3112 \n\
SG#   151 \n\
OPT#    \n\
NAME# P3112 \n\
SGINFO# 151 'P3112 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.66666667 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.33333333 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 152  P3121 \n\
SG#   152 \n\
OPT#    \n\
NAME# P3121 \n\
SGINFO# 152 'P3121 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.66666667 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.33333333 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 153  P3212 \n\
SG#   153 \n\
OPT#    \n\
NAME# P3212 \n\
SGINFO# 153 'P3212 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.33333333 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.66666667 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 154  P3221 \n\
SG#   154 \n\
OPT#    \n\
NAME# P3221 \n\
SGINFO# 154 'P3221 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.33333333 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.66666667 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 155 H R32:H \n\
SG#   155 \n\
OPT#  H  \n\
NAME# R32:H \n\
SGINFO# 155 'R32:H \n\
OPS# 18 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1 -1  0  0 -1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
 -1  0  0 -1  1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  0  1  0  1  0  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  1 -1  0  0 -1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
 -1  0  0 -1  1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  0  1  0  1  0  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 155 R R32:R \n\
SG#   155 \n\
OPT#  R  \n\
NAME# R32:R \n\
SGINFO# 20155 'R32:R \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 156  P3m1 \n\
SG#   156 \n\
OPT#    \n\
NAME# P3m1 \n\
SGINFO# 156 'P3m1 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 157  P31m \n\
SG#   157 \n\
OPT#    \n\
NAME# P31m \n\
SGINFO# 157 'P31m \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 158  P3c1 \n\
SG#   158 \n\
OPT#    \n\
NAME# P3c1 \n\
SGINFO# 158 'P3c1 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 159  P31c \n\
SG#   159 \n\
OPT#    \n\
NAME# P31c \n\
SGINFO# 159 'P31c \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 160 H R3m:H \n\
SG#   160 \n\
OPT#  H  \n\
NAME# R3m:H \n\
SGINFO# 160 'R3m:H \n\
OPS# 18 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  1  0  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 160 R R3m:R \n\
SG#   160 \n\
OPT#  R  \n\
NAME# R3m:R \n\
SGINFO# 20160 'R3m:R \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 161 H R3c:H \n\
SG#   161 \n\
OPT#  H  \n\
NAME# R3c:H \n\
SGINFO# 161 'R3c:H \n\
OPS# 18 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0  0  1  0  0  0  1  0.66666667  0.33333333  0.83333333 \n\
  1  0  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.83333333 \n\
  0 -1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.83333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0  0  1  0  0  0  1  0.33333333  0.66666667  0.16666667 \n\
  1  0  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.16666667 \n\
  0 -1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.16666667 \n\
// **************************************************************** \n\
// SPACEGROUP 161 R R3c:R \n\
SG#   161 \n\
OPT#  R  \n\
NAME# R3c:R \n\
SGINFO# 20161 'R3c:R \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 162  P-31m \n\
SG#   162 \n\
OPT#    \n\
NAME# P-31m \n\
SGINFO# 162 'P-31m \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 163  P-31c \n\
SG#   163 \n\
OPT#    \n\
NAME# P-31c \n\
SGINFO# 163 'P-31c \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 164  P-3m1 \n\
SG#   164 \n\
OPT#    \n\
NAME# P-3m1 \n\
SGINFO# 164 'P-3m1 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 165  P-3c1 \n\
SG#   165 \n\
OPT#    \n\
NAME# P-3c1 \n\
SGINFO# 165 'P-3c1 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 166 H R-3m:H \n\
SG#   166 \n\
OPT#  H  \n\
NAME# R-3m:H \n\
SGINFO# 166 'R-3m:H \n\
OPS# 36 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1 -1  0  0 -1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
 -1  0  0 -1  1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  0  1  0  1  0  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
 -1  0  0  0 -1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  0  1  0 -1  1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  1 -1  0  1  0  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  1 -1  0  0 -1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
 -1  0  0 -1  1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  0  1  0  1  0  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
 -1  0  0  0 -1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  0  1  0 -1  1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  1 -1  0  1  0  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  1  0  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 166 R R-3m:R \n\
SG#   166 \n\
OPT#  R  \n\
NAME# R-3m:R \n\
SGINFO# 20166 'R-3m:R \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 167 H R-3c:H \n\
SG#   167 \n\
OPT#  H  \n\
NAME# R-3c:H \n\
SGINFO# 167 'R-3c:H \n\
OPS# 36 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.33333333 \n\
  1 -1  0  0 -1  0  0  0 -1  0.66666667  0.33333333  0.83333333 \n\
 -1  0  0 -1  1  0  0  0 -1  0.66666667  0.33333333  0.83333333 \n\
  0  1  0  1  0  0  0  0 -1  0.66666667  0.33333333  0.83333333 \n\
 -1  0  0  0 -1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  0  1  0 -1  1  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
  1 -1  0  1  0  0  0  0 -1  0.66666667  0.33333333  0.33333333 \n\
 -1  1  0  0  1  0  0  0  1  0.66666667  0.33333333  0.83333333 \n\
  1  0  0  1 -1  0  0  0  1  0.66666667  0.33333333  0.83333333 \n\
  0 -1  0 -1  0  0  0  0  1  0.66666667  0.33333333  0.83333333 \n\
  1  0  0  0  1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.66666667 \n\
  1 -1  0  0 -1  0  0  0 -1  0.33333333  0.66666667  0.16666667 \n\
 -1  0  0 -1  1  0  0  0 -1  0.33333333  0.66666667  0.16666667 \n\
  0  1  0  1  0  0  0  0 -1  0.33333333  0.66666667  0.16666667 \n\
 -1  0  0  0 -1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  0  1  0 -1  1  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
  1 -1  0  1  0  0  0  0 -1  0.33333333  0.66666667  0.66666667 \n\
 -1  1  0  0  1  0  0  0  1  0.33333333  0.66666667  0.16666667 \n\
  1  0  0  1 -1  0  0  0  1  0.33333333  0.66666667  0.16666667 \n\
  0 -1  0 -1  0  0  0  0  1  0.33333333  0.66666667  0.16666667 \n\
// **************************************************************** \n\
// SPACEGROUP 167 R R-3c:R \n\
SG#   167 \n\
OPT#  R  \n\
NAME# R-3c:R \n\
SGINFO# 20167 'R-3c:R \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 168  P6 \n\
SG#   168 \n\
OPT#    \n\
NAME# P6 \n\
SGINFO# 168 'P6 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 169  P61 \n\
SG#   169 \n\
OPT#    \n\
NAME# P61 \n\
SGINFO# 169 'P61 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.16666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.83333333 \n\
// **************************************************************** \n\
// SPACEGROUP 170  P65 \n\
SG#   170 \n\
OPT#    \n\
NAME# P65 \n\
SGINFO# 170 'P65 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.83333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.16666667 \n\
// **************************************************************** \n\
// SPACEGROUP 171  P62 \n\
SG#   171 \n\
OPT#    \n\
NAME# P62 \n\
SGINFO# 171 'P62 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 172  P64 \n\
SG#   172 \n\
OPT#    \n\
NAME# P64 \n\
SGINFO# 172 'P64 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.33333333 \n\
// **************************************************************** \n\
// SPACEGROUP 173  P63 \n\
SG#   173 \n\
OPT#    \n\
NAME# P63 \n\
SGINFO# 173 'P63 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 174  P-6 \n\
SG#   174 \n\
OPT#    \n\
NAME# P-6 \n\
SGINFO# 174 'P-6 \n\
OPS# 6 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 175  P6/m \n\
SG#   175 \n\
OPT#    \n\
NAME# P6/m \n\
SGINFO# 175 'P6/m \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 176  P63/m \n\
SG#   176 \n\
OPT#    \n\
NAME# P63/m \n\
SGINFO# 176 'P63/m \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 177  P622 \n\
SG#   177 \n\
OPT#    \n\
NAME# P622 \n\
SGINFO# 177 'P622 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 178  P6122 \n\
SG#   178 \n\
OPT#    \n\
NAME# P6122 \n\
SGINFO# 178 'P6122 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.16666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.83333333 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.66666667 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.33333333 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.83333333 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.16666667 \n\
// **************************************************************** \n\
// SPACEGROUP 179  P6522 \n\
SG#   179 \n\
OPT#    \n\
NAME# P6522 \n\
SGINFO# 179 'P6522 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.83333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.16666667 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.33333333 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.66666667 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.16666667 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.83333333 \n\
// **************************************************************** \n\
// SPACEGROUP 180  P6222 \n\
SG#   180 \n\
OPT#    \n\
NAME# P6222 \n\
SGINFO# 180 'P6222 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.66666667 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.33333333 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.66666667 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.33333333 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.66666667 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.66666667 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.33333333 \n\
// **************************************************************** \n\
// SPACEGROUP 181  P6422 \n\
SG#   181 \n\
OPT#    \n\
NAME# P6422 \n\
SGINFO# 181 'P6422 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.33333333 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.66666667 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.33333333 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.66666667 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.33333333 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.33333333 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.66666667 \n\
// **************************************************************** \n\
// SPACEGROUP 182  P6322 \n\
SG#   182 \n\
OPT#    \n\
NAME# P6322 \n\
SGINFO# 182 'P6322 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 183  P6mm \n\
SG#   183 \n\
OPT#    \n\
NAME# P6mm \n\
SGINFO# 183 'P6mm \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 184  P6cc \n\
SG#   184 \n\
OPT#    \n\
NAME# P6cc \n\
SGINFO# 184 'P6cc \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 185  P63cm \n\
SG#   185 \n\
OPT#    \n\
NAME# P63cm \n\
SGINFO# 185 'P63cm \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 186  P63mc \n\
SG#   186 \n\
OPT#    \n\
NAME# P63mc \n\
SGINFO# 186 'P63mc \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 187  P-6m2 \n\
SG#   187 \n\
OPT#    \n\
NAME# P-6m2 \n\
SGINFO# 187 'P-6m2 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 188  P-6c2 \n\
SG#   188 \n\
OPT#    \n\
NAME# P-6c2 \n\
SGINFO# 188 'P-6c2 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 189  P-62m \n\
SG#   189 \n\
OPT#    \n\
NAME# P-62m \n\
SGINFO# 189 'P-62m \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 190  P-62c \n\
SG#   190 \n\
OPT#    \n\
NAME# P-62c \n\
SGINFO# 190 'P-62c \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 191  P6/mmm \n\
SG#   191 \n\
OPT#    \n\
NAME# P6/mmm \n\
SGINFO# 191 'P6/mmm \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 192  P6/mcc \n\
SG#   192 \n\
OPT#    \n\
NAME# P6/mcc \n\
SGINFO# 192 'P6/mcc \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 193  P63/mcm \n\
SG#   193 \n\
OPT#    \n\
NAME# P63/mcm \n\
SGINFO# 193 'P63/mcm \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 194  P63/mmc \n\
SG#   194 \n\
OPT#    \n\
NAME# P63/mmc \n\
SGINFO# 194 'P63/mmc \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0 -1  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0  1  0 -1  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.50 \n\
  1 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  1  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  1 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1 -1  0  0 -1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0 -1  1  0  0  0  1  0.00  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 195  P23 \n\
SG#   195 \n\
OPT#    \n\
NAME# P23 \n\
SGINFO# 195 'P23 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 196  F23 \n\
SG#   196 \n\
OPT#    \n\
NAME# F23 \n\
SGINFO# 196 'F23 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 197  I23 \n\
SG#   197 \n\
OPT#    \n\
NAME# I23 \n\
SGINFO# 197 'I23 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 198  P213 \n\
SG#   198 \n\
OPT#    \n\
NAME# P213 \n\
SGINFO# 198 'P213 \n\
OPS# 12 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 199  I213 \n\
SG#   199 \n\
OPT#    \n\
NAME# I213 \n\
SGINFO# 199 'I213 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 200  Pm-3 \n\
SG#   200 \n\
OPT#    \n\
NAME# Pm-3 \n\
SGINFO# 200 'Pm-3 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 201 1 Pn-3:1 \n\
SG#   201 \n\
OPT#  1  \n\
NAME# Pn-3:1 \n\
SGINFO# 201 'Pn-3:1 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 201 2 Pn-3:2 \n\
SG#   201 \n\
OPT#  2  \n\
NAME# Pn-3:2 \n\
SGINFO# 20201 'Pn-3:2 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 202  Fm-3 \n\
SG#   202 \n\
OPT#    \n\
NAME# Fm-3 \n\
SGINFO# 202 'Fm-3 \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 203 1 Fd-3:1 \n\
SG#   203 \n\
OPT#  1  \n\
NAME# Fd-3:1 \n\
SGINFO# 203 'Fd-3:1 \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.25  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.25  0.25  0.25 \n\
  0 -1  0  0  0 -1 -1  0  0  0.25  0.25  0.25 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.25  0.25 \n\
  0  0 -1  1  0  0  0  1  0  0.25  0.25  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.25  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.25  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.75  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.25  0.75  0.75 \n\
  0 -1  0  0  0 -1 -1  0  0  0.25  0.75  0.75 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.75  0.75 \n\
  0  0 -1  1  0  0  0  1  0  0.25  0.75  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.75  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.75  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.75  0.25  0.75 \n\
  0 -1  0  0  0 -1 -1  0  0  0.75  0.25  0.75 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.25  0.75 \n\
  0  0 -1  1  0  0  0  1  0  0.75  0.25  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.25  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.75  0.75  0.25 \n\
  0 -1  0  0  0 -1 -1  0  0  0.75  0.75  0.25 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  1  0  0  0  1  0  0.75  0.75  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.75  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.75  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.75  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.75  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 203 2 Fd-3:2 \n\
SG#   203 \n\
OPT#  2  \n\
NAME# Fd-3:2 \n\
SGINFO# 20203 'Fd-3:2 \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.25  0.25  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.25  0.25 \n\
  0 -1  0  0  0  1 -1  0  0  0.25  0.00  0.25 \n\
  0  0 -1 -1  0  0  0  1  0  0.25  0.25  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.25  0.00  0.25 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.25  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.00  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.75  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.75  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.00  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.75  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.00  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.75  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.00  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.25  0.75  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.75  0.75 \n\
  0 -1  0  0  0  1 -1  0  0  0.25  0.50  0.75 \n\
  0  0 -1 -1  0  0  0  1  0  0.25  0.75  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.25  0.50  0.75 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.75  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.50  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.25  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.25  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.50  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.25  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.50  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.25  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.50  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.75  0.25  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.25  0.75 \n\
  0 -1  0  0  0  1 -1  0  0  0.75  0.00  0.75 \n\
  0  0 -1 -1  0  0  0  1  0  0.75  0.25  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.75  0.00  0.75 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.25  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.00  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.75  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.75  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.00  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.75  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.00  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.00  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.75  0.75  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.75  0.25 \n\
  0 -1  0  0  0  1 -1  0  0  0.75  0.50  0.25 \n\
  0  0 -1 -1  0  0  0  1  0  0.75  0.75  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.75  0.50  0.25 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.75  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.50  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.25  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.25  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.50  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.25  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.50  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.50  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 204  Im-3 \n\
SG#   204 \n\
OPT#    \n\
NAME# Im-3 \n\
SGINFO# 204 'Im-3 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 205  Pa-3 \n\
SG#   205 \n\
OPT#    \n\
NAME# Pa-3 \n\
SGINFO# 205 'Pa-3 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 206  Ia-3 \n\
SG#   206 \n\
OPT#    \n\
NAME# Ia-3 \n\
SGINFO# 206 'Ia-3 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 207  P432 \n\
SG#   207 \n\
OPT#    \n\
NAME# P432 \n\
SGINFO# 207 'P432 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 208  P4232 \n\
SG#   208 \n\
OPT#    \n\
NAME# P4232 \n\
SGINFO# 208 'P4232 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 209  F432 \n\
SG#   209 \n\
OPT#    \n\
NAME# F432 \n\
SGINFO# 209 'F432 \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 210  F4132 \n\
SG#   210 \n\
OPT#    \n\
NAME# F4132 \n\
SGINFO# 210 'F4132 \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.25  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.25  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.25  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.75  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.75  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.75  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.75  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 211  I432 \n\
SG#   211 \n\
OPT#    \n\
NAME# I432 \n\
SGINFO# 211 'I432 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 212  P4332 \n\
SG#   212 \n\
OPT#    \n\
NAME# P4332 \n\
SGINFO# 212 'P4332 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.25  0.25 \n\
// **************************************************************** \n\
// SPACEGROUP 213  P4132 \n\
SG#   213 \n\
OPT#    \n\
NAME# P4132 \n\
SGINFO# 213 'P4132 \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.75  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.25  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.75  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 214  I4132 \n\
SG#   214 \n\
OPT#    \n\
NAME# I4132 \n\
SGINFO# 214 'I4132 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.25  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.75  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.75  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 215  P-43m \n\
SG#   215 \n\
OPT#    \n\
NAME# P-43m \n\
SGINFO# 215 'P-43m \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 216  F-43m \n\
SG#   216 \n\
OPT#    \n\
NAME# F-43m \n\
SGINFO# 216 'F-43m \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 217  I-43m \n\
SG#   217 \n\
OPT#    \n\
NAME# I-43m \n\
SGINFO# 217 'I-43m \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 218  P-43n \n\
SG#   218 \n\
OPT#    \n\
NAME# P-43n \n\
SGINFO# 218 'P-43n \n\
OPS# 24 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 219  F-43c \n\
SG#   219 \n\
OPT#    \n\
NAME# F-43c \n\
SGINFO# 219 'F-43c \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 220  I-43d \n\
SG#   220 \n\
OPT#    \n\
NAME# I-43d \n\
SGINFO# 220 'I-43d \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.25  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.25  0.75 \n\
 -1  0  0  0  0  1  0 -1  0  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.75  0.25  0.25 \n\
  0  0 -1  0 -1  0  1  0  0  0.75  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.25  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.25  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  0 -1  0 -1  0  0.25  0.75  0.25 \n\
  1  0  0  0  0  1  0  1  0  0.25  0.25  0.25 \n\
  0  0 -1  0  1  0 -1  0  0  0.25  0.25  0.75 \n\
  0  0  1  0  1  0  1  0  0  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  0  1  0 -1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.25  0.75  0.75 \n\
  0  0 -1  0 -1  0  1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.75  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.75  0.75  0.75 \n\
  1  0  0  0  0 -1  0 -1  0  0.75  0.25  0.75 \n\
  1  0  0  0  0  1  0  1  0  0.75  0.75  0.75 \n\
  0  0 -1  0  1  0 -1  0  0  0.75  0.75  0.25 \n\
  0  0  1  0  1  0  1  0  0  0.75  0.75  0.75 \n\
// **************************************************************** \n\
// SPACEGROUP 221  Pm-3m \n\
SG#   221 \n\
OPT#    \n\
NAME# Pm-3m \n\
SGINFO# 221 'Pm-3m \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 222 1 Pn-3n:1 \n\
SG#   222 \n\
OPT#  1  \n\
NAME# Pn-3n:1 \n\
SGINFO# 222 'Pn-3n:1 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 222 2 Pn-3n:2 \n\
SG#   222 \n\
OPT#  2  \n\
NAME# Pn-3n:2 \n\
SGINFO# 20222 'Pn-3n:2 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.50  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 223  Pm-3n \n\
SG#   223 \n\
OPT#    \n\
NAME# Pm-3n \n\
SGINFO# 223 'Pm-3n \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 224 1 Pn-3m:1 \n\
SG#   224 \n\
OPT#  1  \n\
NAME# Pn-3m:1 \n\
SGINFO# 224 'Pn-3m:1 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 224 2 Pn-3m:2 \n\
SG#   224 \n\
OPT#  2  \n\
NAME# Pn-3m:2 \n\
SGINFO# 20224 'Pn-3m:2 \n\
OPS# 48 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 225  Fm-3m \n\
SG#   225 \n\
OPT#    \n\
NAME# Fm-3m \n\
SGINFO# 225 'Fm-3m \n\
OPS# 192 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 226  Fm-3c \n\
SG#   226 \n\
OPT#    \n\
NAME# Fm-3c \n\
SGINFO# 226 'Fm-3c \n\
OPS# 192 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.50  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.50  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.50  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.50  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 227 1 Fd-3m:1 \n\
SG#   227 \n\
OPT#  1  \n\
NAME# Fd-3m:1 \n\
SGINFO# 227 'Fd-3m:1 \n\
OPS# 192 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.25  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.25  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.25  0.25  0.25 \n\
  0 -1  0  0  0 -1 -1  0  0  0.25  0.25  0.25 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.25  0.25 \n\
  0  0 -1  1  0  0  0  1  0  0.25  0.25  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.25  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.25  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.25  0.25  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.25  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.25  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.75  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.25  0.75  0.75 \n\
  0 -1  0  0  0 -1 -1  0  0  0.25  0.75  0.75 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.75  0.75 \n\
  0  0 -1  1  0  0  0  1  0  0.25  0.75  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.75  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.75  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.25  0.75  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.75  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.75  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.25  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.25  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.75  0.25  0.75 \n\
  0 -1  0  0  0 -1 -1  0  0  0.75  0.25  0.75 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.25  0.75 \n\
  0  0 -1  1  0  0  0  1  0  0.75  0.25  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.25  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.75  0.25  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.75  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.75  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.75  0.75  0.25 \n\
  0 -1  0  0  0 -1 -1  0  0  0.75  0.75  0.25 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  1  0  0  0  1  0  0.75  0.75  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.75  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.75  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.75  0.75  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 227 2 Fd-3m:2 \n\
SG#   227 \n\
OPT#  2  \n\
NAME# Fd-3m:2 \n\
SGINFO# 20227 'Fd-3m:2 \n\
OPS# 192 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.25  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.00  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.00  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.25  0.25 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.25  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.25  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.00  0.25 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.25  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.25  0.25  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.25  0.25 \n\
  0 -1  0  0  0  1 -1  0  0  0.25  0.00  0.25 \n\
  0  0 -1 -1  0  0  0  1  0  0.25  0.25  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.25  0.00  0.25 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.25  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.25  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.25  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.00  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.75  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.00  0.75 \n\
 -1  0  0  0  0  1  0 -1  0  0.75  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.75  0.75 \n\
 -1  0  0  0  0 -1  0  1  0  0.75  0.75  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.75  0.75  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.00  0.75 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.75  0.75 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.75  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.75  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.00  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.75  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.00  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.75  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.75  0.75 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.75  0.00  0.75 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.75  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.50  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.50  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.75  0.75 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.50  0.75 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.75  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.25  0.75  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.75  0.75 \n\
  0 -1  0  0  0  1 -1  0  0  0.25  0.50  0.75 \n\
  0  0 -1 -1  0  0  0  1  0  0.25  0.75  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.25  0.50  0.75 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.75  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.75  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.50  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.25  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.50  0.25 \n\
 -1  0  0  0  0  1  0 -1  0  0.75  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.25  0.25 \n\
 -1  0  0  0  0 -1  0  1  0  0.75  0.25  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.75  0.25  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.50  0.25 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.25  0.25 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.25  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.25  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.50  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.25  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.50  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.25  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.25  0.25 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.75  0.50  0.25 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.25  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.00  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.00  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.25  0.75 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.00  0.75 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.75  0.25  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.25  0.75 \n\
  0 -1  0  0  0  1 -1  0  0  0.75  0.00  0.75 \n\
  0  0 -1 -1  0  0  0  1  0  0.75  0.25  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.75  0.00  0.75 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.25  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.00  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.00  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.00  0.25 \n\
 -1  0  0  0  0  1  0 -1  0  0.25  0.00  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.75  0.25 \n\
 -1  0  0  0  0 -1  0  1  0  0.25  0.75  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.25  0.75  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.00  0.25 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.75  0.25 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.75  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.75  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.00  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.75  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.00  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.75  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.75  0.25 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.25  0.00  0.25 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.75  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.50  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.50  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.75  0.25 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.75  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.75  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.50  0.25 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.75  0.75  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.75  0.25 \n\
  0 -1  0  0  0  1 -1  0  0  0.75  0.50  0.25 \n\
  0  0 -1 -1  0  0  0  1  0  0.75  0.75  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.75  0.50  0.25 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.75  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.75  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.50  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.50  0.75 \n\
 -1  0  0  0  0  1  0 -1  0  0.25  0.50  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.25  0.75 \n\
 -1  0  0  0  0 -1  0  1  0  0.25  0.25  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.25  0.25  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.50  0.75 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.25  0.75 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.25  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.25  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.50  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.25  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.50  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.25  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.25  0.75 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.25  0.50  0.75 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.00 \n\
// **************************************************************** \n\
// SPACEGROUP 228 1 Fd-3c:1 \n\
SG#   228 \n\
OPT#  1  \n\
NAME# Fd-3c:1 \n\
SGINFO# 228 'Fd-3c:1 \n\
OPS# 192 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.25  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.25  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.25  0.25  0.75 \n\
  0 -1  0  0  0 -1 -1  0  0  0.25  0.25  0.75 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.25  0.75 \n\
  0  0 -1  1  0  0  0  1  0  0.25  0.25  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.25  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.25  0.75 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.25  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.25  0.25  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.25  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.25  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.25  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.25  0.75  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.75  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.75  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.50  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.25  0.75  0.25 \n\
  0 -1  0  0  0 -1 -1  0  0  0.25  0.75  0.25 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.75  0.25 \n\
  0  0 -1  1  0  0  0  1  0  0.25  0.75  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.75  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.75  0.25 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.75  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.25  0.75  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.75  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.75  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.75  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.75  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.25  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.50  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.25  0.25 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.25  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.25  0.75 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.75  0.25  0.25 \n\
  0 -1  0  0  0 -1 -1  0  0  0.75  0.25  0.25 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.25  0.25 \n\
  0  0 -1  1  0  0  0  1  0  0.75  0.25  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.25  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.25  0.25 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.25  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.75  0.25  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.25  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.75  0.75  0.75 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.75  0.25 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.75  0.75  0.75 \n\
  0 -1  0  0  0 -1 -1  0  0  0.75  0.75  0.75 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.75  0.75 \n\
  0  0 -1  1  0  0  0  1  0  0.75  0.75  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.75  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.75  0.75 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.75  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.75  0.75  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.75  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.75  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 228 2 Fd-3c:2 \n\
SG#   228 \n\
OPT#  2  \n\
NAME# Fd-3c:2 \n\
SGINFO# 20228 'Fd-3c:2 \n\
OPS# 192 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.25  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.00  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.00  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.25  0.25 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.00  0.25 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.25  0.25  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.25  0.25 \n\
  0 -1  0  0  0  1 -1  0  0  0.25  0.00  0.25 \n\
  0  0 -1 -1  0  0  0  1  0  0.25  0.25  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.25  0.00  0.25 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.25  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.00  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.75  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.00  0.25 \n\
 -1  0  0  0  0  1  0 -1  0  0.75  0.00  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.75  0.75 \n\
 -1  0  0  0  0 -1  0  1  0  0.75  0.25  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.75  0.25  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.00  0.75 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.75  0.25 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.75  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.75  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.00  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.75  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.00  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.25  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.75  0.25 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.75  0.00  0.25 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.50 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.25  0.75  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.50  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.50  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.75  0.75 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.25  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.25  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.25  0.50  0.75 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.25  0.75  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.75  0.75 \n\
  0 -1  0  0  0  1 -1  0  0  0.25  0.50  0.75 \n\
  0  0 -1 -1  0  0  0  1  0  0.25  0.75  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.25  0.50  0.75 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.75  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.25  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.50  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.50  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.50  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.50  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.75  0.25  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.50  0.75 \n\
 -1  0  0  0  0  1  0 -1  0  0.75  0.50  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.25  0.25 \n\
 -1  0  0  0  0 -1  0  1  0  0.75  0.75  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.75  0.75  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.75  0.50  0.25 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.25  0.75 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.75  0.25  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.25  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.75  0.50  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.75  0.25  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.75  0.50  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.75  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.50  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.25  0.75 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.75  0.50  0.75 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.50  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.25  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.00  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.00  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.25  0.75 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.75  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.75  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.00  0.75 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.25  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.75  0.25  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.25  0.75 \n\
  0 -1  0  0  0  1 -1  0  0  0.75  0.00  0.75 \n\
  0  0 -1 -1  0  0  0  1  0  0.75  0.25  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.75  0.00  0.75 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.25  0.75 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.75  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.25  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.00  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.75  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.75  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.00  0.75 \n\
 -1  0  0  0  0  1  0 -1  0  0.25  0.00  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.75  0.25 \n\
 -1  0  0  0  0 -1  0  1  0  0.25  0.25  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.25  0.25  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.00  0.25 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.75  0.75 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.00  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.75  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.75  0.25 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.00  0.25 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.75  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.00  0.25 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.75  0.25 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.25  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.75  0.75 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.25  0.00  0.75 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.75  0.75  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.50  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.50  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.75  0.25 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.75  0.50  0.25 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.75  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.75  0.75  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.75  0.25 \n\
  0 -1  0  0  0  1 -1  0  0  0.75  0.50  0.25 \n\
  0  0 -1 -1  0  0  0  1  0  0.75  0.75  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.75  0.50  0.25 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.75  0.25 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.75  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.50  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.25  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.25  0.25  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.50  0.25 \n\
 -1  0  0  0  0  1  0 -1  0  0.25  0.50  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.25  0.75 \n\
 -1  0  0  0  0 -1  0  1  0  0.25  0.75  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.25  0.75  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.25  0.50  0.75 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.25  0.25 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.25  0.25  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.25  0.75 \n\
  0  1  0  0  0 -1  1  0  0  0.25  0.50  0.75 \n\
  0  0  1  1  0  0  0 -1  0  0.25  0.25  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.25  0.50  0.75 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.25  0.75 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.75  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.25  0.25 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.25  0.50  0.25 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 229  Im-3m \n\
SG#   229 \n\
OPT#    \n\
NAME# Im-3m \n\
SGINFO# 229 'Im-3m \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.00  0.00  0.00 \n\
 -1  0  0  0  0  1  0 -1  0  0.00  0.00  0.00 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0 -1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.00  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.00  0.00  0.00 \n\
  0  1  0  1  0  0  0  0  1  0.00  0.00  0.00 \n\
  1  0  0  0  0 -1  0 -1  0  0.00  0.00  0.00 \n\
  1  0  0  0  0  1  0  1  0  0.00  0.00  0.00 \n\
  0  0 -1  0  1  0 -1  0  0  0.00  0.00  0.00 \n\
  0  0  1  0  1  0  1  0  0  0.00  0.00  0.00 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.50  0.50  0.50 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.50  0.50  0.50 \n\
 -1  0  0  0  0  1  0 -1  0  0.50  0.50  0.50 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0 -1  0  1  0  0  0.50  0.50  0.50 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.50  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.50  0.50  0.50 \n\
  0  1  0  1  0  0  0  0  1  0.50  0.50  0.50 \n\
  1  0  0  0  0 -1  0 -1  0  0.50  0.50  0.50 \n\
  1  0  0  0  0  1  0  1  0  0.50  0.50  0.50 \n\
  0  0 -1  0  1  0 -1  0  0  0.50  0.50  0.50 \n\
  0  0  1  0  1  0  1  0  0  0.50  0.50  0.50 \n\
// **************************************************************** \n\
// SPACEGROUP 230  Ia-3d \n\
SG#   230 \n\
OPT#    \n\
NAME# Ia-3d \n\
SGINFO# 230 'Ia-3d \n\
OPS# 96 \n\
  1  0  0  0  1  0  0  0  1  0.00  0.00  0.00 \n\
  0 -1  0  1  0  0  0  0  1  0.25  0.75  0.25 \n\
 -1  0  0  0 -1  0  0  0  1  0.00  0.50  0.00 \n\
  0  1  0 -1  0  0  0  0  1  0.25  0.25  0.75 \n\
  1  0  0  0  0 -1  0  1  0  0.25  0.25  0.75 \n\
  1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.50 \n\
  1  0  0  0  0  1  0 -1  0  0.75  0.25  0.25 \n\
  0  0  1  0  1  0 -1  0  0  0.75  0.25  0.25 \n\
 -1  0  0  0  1  0  0  0 -1  0.50  0.00  0.00 \n\
  0  0 -1  0  1  0  1  0  0  0.25  0.75  0.25 \n\
  0  0  1  1  0  0  0  1  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1  1  0  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1  1  0  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0 -1  0  0.00  0.00  0.50 \n\
  0 -1  0  0  0  1 -1  0  0  0.50  0.00  0.00 \n\
  0  0 -1 -1  0  0  0  1  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0 -1  0  0.50  0.00  0.00 \n\
  0  1  0  0  0 -1 -1  0  0  0.00  0.00  0.50 \n\
  0  1  0  1  0  0  0  0 -1  0.75  0.25  0.25 \n\
  0 -1  0 -1  0  0  0  0 -1  0.25  0.25  0.25 \n\
 -1  0  0  0  0  1  0  1  0  0.25  0.75  0.25 \n\
 -1  0  0  0  0 -1  0 -1  0  0.25  0.25  0.25 \n\
  0  0  1  0 -1  0  1  0  0  0.25  0.25  0.75 \n\
  0  0 -1  0 -1  0 -1  0  0  0.25  0.25  0.25 \n\
 -1  0  0  0 -1  0  0  0 -1  0.00  0.00  0.00 \n\
  0  1  0 -1  0  0  0  0 -1  0.75  0.25  0.75 \n\
  1  0  0  0  1  0  0  0 -1  0.00  0.50  0.00 \n\
  0 -1  0  1  0  0  0  0 -1  0.75  0.75  0.25 \n\
 -1  0  0  0  0  1  0 -1  0  0.75  0.75  0.25 \n\
 -1  0  0  0  1  0  0  0  1  0.00  0.00  0.50 \n\
 -1  0  0  0  0 -1  0  1  0  0.25  0.75  0.75 \n\
  0  0 -1  0 -1  0  1  0  0  0.25  0.75  0.75 \n\
  1  0  0  0 -1  0  0  0  1  0.50  0.00  0.00 \n\
  0  0  1  0 -1  0 -1  0  0  0.75  0.25  0.75 \n\
  0  0 -1 -1  0  0  0 -1  0  0.00  0.00  0.00 \n\
  0 -1  0  0  0 -1 -1  0  0  0.00  0.00  0.00 \n\
  0  1  0  0  0  1 -1  0  0  0.00  0.50  0.00 \n\
  0  0 -1  1  0  0  0  1  0  0.00  0.00  0.50 \n\
  0  1  0  0  0 -1  1  0  0  0.50  0.00  0.00 \n\
  0  0  1  1  0  0  0 -1  0  0.00  0.50  0.00 \n\
  0  0  1 -1  0  0  0  1  0  0.50  0.00  0.00 \n\
  0 -1  0  0  0  1  1  0  0  0.00  0.00  0.50 \n\
  0 -1  0 -1  0  0  0  0  1  0.25  0.75  0.75 \n\
  0  1  0  1  0  0  0  0  1  0.75  0.75  0.75 \n\
  1  0  0  0  0 -1  0 -1  0  0.75  0.25  0.75 \n\
  1  0  0  0  0  1  0  1  0  0.75  0.75  0.75 \n\
  0  0 -1  0  1  0 -1  0  0  0.75  0.75  0.25 \n\
  0  0  1  0  1  0  1  0  0  0.75  0.75  0.75 \n\
  1  0  0  0  1  0  0  0  1  0.50  0.50  0.50 \n\
  0 -1  0  1  0  0  0  0  1  0.75  0.25  0.75 \n\
 -1  0  0  0 -1  0  0  0  1  0.50  0.00  0.50 \n\
  0  1  0 -1  0  0  0  0  1  0.75  0.75  0.25 \n\
  1  0  0  0  0 -1  0  1  0  0.75  0.75  0.25 \n\
  1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.00 \n\
  1  0  0  0  0  1  0 -1  0  0.25  0.75  0.75 \n\
  0  0  1  0  1  0 -1  0  0  0.25  0.75  0.75 \n\
 -1  0  0  0  1  0  0  0 -1  0.00  0.50  0.50 \n\
  0  0 -1  0  1  0  1  0  0  0.75  0.25  0.75 \n\
  0  0  1  1  0  0  0  1  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1  1  0  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1  1  0  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0 -1  0  0.50  0.50  0.00 \n\
  0 -1  0  0  0  1 -1  0  0  0.00  0.50  0.50 \n\
  0  0 -1 -1  0  0  0  1  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0 -1  0  0.00  0.50  0.50 \n\
  0  1  0  0  0 -1 -1  0  0  0.50  0.50  0.00 \n\
  0  1  0  1  0  0  0  0 -1  0.25  0.75  0.75 \n\
  0 -1  0 -1  0  0  0  0 -1  0.75  0.75  0.75 \n\
 -1  0  0  0  0  1  0  1  0  0.75  0.25  0.75 \n\
 -1  0  0  0  0 -1  0 -1  0  0.75  0.75  0.75 \n\
  0  0  1  0 -1  0  1  0  0  0.75  0.75  0.25 \n\
  0  0 -1  0 -1  0 -1  0  0  0.75  0.75  0.75 \n\
 -1  0  0  0 -1  0  0  0 -1  0.50  0.50  0.50 \n\
  0  1  0 -1  0  0  0  0 -1  0.25  0.75  0.25 \n\
  1  0  0  0  1  0  0  0 -1  0.50  0.00  0.50 \n\
  0 -1  0  1  0  0  0  0 -1  0.25  0.25  0.75 \n\
 -1  0  0  0  0  1  0 -1  0  0.25  0.25  0.75 \n\
 -1  0  0  0  1  0  0  0  1  0.50  0.50  0.00 \n\
 -1  0  0  0  0 -1  0  1  0  0.75  0.25  0.25 \n\
  0  0 -1  0 -1  0  1  0  0  0.75  0.25  0.25 \n\
  1  0  0  0 -1  0  0  0  1  0.00  0.50  0.50 \n\
  0  0  1  0 -1  0 -1  0  0  0.25  0.75  0.25 \n\
  0  0 -1 -1  0  0  0 -1  0  0.50  0.50  0.50 \n\
  0 -1  0  0  0 -1 -1  0  0  0.50  0.50  0.50 \n\
  0  1  0  0  0  1 -1  0  0  0.50  0.00  0.50 \n\
  0  0 -1  1  0  0  0  1  0  0.50  0.50  0.00 \n\
  0  1  0  0  0 -1  1  0  0  0.00  0.50  0.50 \n\
  0  0  1  1  0  0  0 -1  0  0.50  0.00  0.50 \n\
  0  0  1 -1  0  0  0  1  0  0.00  0.50  0.50 \n\
  0 -1  0  0  0  1  1  0  0  0.50  0.50  0.00 \n\
  0 -1  0 -1  0  0  0  0  1  0.75  0.25  0.25 \n\
  0  1  0  1  0  0  0  0  1  0.25  0.25  0.25 \n\
  1  0  0  0  0 -1  0 -1  0  0.25  0.75  0.25 \n\
  1  0  0  0  0  1  0  1  0  0.25  0.25  0.25 \n\
  0  0 -1  0  1  0 -1  0  0  0.25  0.25  0.75 \n\
  0  0  1  0  1  0  1  0  0  0.25  0.25  0.25 \n\
";

// ***************************************************************************

#endif

// ***************************************************************************
// *                                                                         *
// *           Aflow STEFANO CURTAROLO - Duke University 2003-2018           *
// *                                                                         *
// ***************************************************************************
// Stefano Curtarolo, 2009-2012 Duke


